#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id$
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import math, StringIO, string, new, sys, inspect
import scipy.weave, scipy.optimize, scipy.odr.odrpack, scipy.stats,  numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them

import ExtraCodeForEquationBaseClasses # offloaded some code into this file to make maintenence easier


class Equation:

    ascii = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ' # used in polynomials and polyfunctionals
    FittingTargetDict = {'SSQABS':['Fitting target is lowest sum of squared (SSQ) absolute error (traditional)', 'sum of squared absolute error'],
                         'ODR':['Fitting target is lowest sum of orthogonal distance (ODR) (robust, slow)', 'sum of orthogonal distance (ODR)'],
                         'SSQREL':['Fitting target is lowest sum of squared (SSQ) relative error (robust, slow)', 'sum of squared relative error'],
                         'ABSABS':['Fitting target is lowest sum of absolute value of absolute error (robust, slow)', 'sum of absolute value of absolute error'],
                         'ABSREL':['Fitting target is lowest sum of absolute value of relative error (robust, slow)', 'sum of absolute value of relative error'],
                         'PEAKABS':['Fitting target is lowest peak absolute value of absolute error (robust, slow)', 'peak absolute value of absolute error'],
                         'PEAKREL':['Fitting target is lowest peak absolute value of relative error (robust, slow)', 'peak absolute value of relative error'],
                         'AIC':['Fitting target is lowest AIC (robust, slow)', 'AIC'],
                         'BIC':['Fitting target is lowest BIC (robust, slow)', 'BIC']
                        }

    # for weave.inline() C++ compilation under GCC 4.2, change as needed for your CPU and compiler
    if sys.platform == 'win32': # try to pre-detect MSVC++ or GCC and optimize appropiately
        compiler_flags_for_weave_inline = ['/O2']
    else:
        compiler_flags_for_weave_inline = ['-march=native', '-O2', '-mfpmath=sse']

    def __init__(self):
        self.splineFlag = 0
        self.nonlinearSolver = 1 # 1 = fmin, 2 = fmin_bfgs, 3 = fmin_cg, 4 = fmin_ncq, 5 = fmin_powell
        self.polyfunctionalFlag = 0
        self.polynomialFlag = 0
        self.LinearSSQSolverFlag = 0
        self.RequiresNonzeroX = 0
        self.RequiresNonzeroY = 0
        self.RequiresPositiveX = 0
        self.RequiresPositiveY = 0
        self.RequiresNegativeX = 0
        self.RequiresNegativeY = 0

        self.coefficientArray = []
        self.fixedCoefficientArray = []
        
        # let the C++ in weave know if the equation needs a bounds check on coefficients
        self.requiresCoefficientBoundsCheckWhenFitting = 0

        # always start with an empty cache dictionary, this
        # holds both the "reduced data" and "all data" caches
        self.cacheDict = {}

        # having these empty/zero defaults makes saving equation parameters easier
        # in the function finders, even though most equations do not use them
        self.Polyfun2DFlags = []
        self.Polyfun3DFlags = []
        self.xPolynomialOrder = 0
        self.yPolynomialOrder = 0

        # for HTML output
        self.HTMLIsSplitIntoTwoLinesOfHTML = 0
        self.DependentDataContainsZeroFlag = False

        # for decoupling weave cache length from coefficient array length
        self.CacheGenerationList = []

        # default GA parameters
        self.fmin_xtol = 1.0E-10
        self.fmin_ftol = 1.0E-10
        self.fminIterationLimit = 1000
        self.fminFunctionLimit = 1000
        self.guessDivisor = 5.0
        self.diffScale = 0.7
        self.crossoverProb = 0.7
        self.maxparamvalue = 100.0
        self.minparamvalue = -100.0

        self.extendedForm = False # extended form of the equation? set in CommonInitCodeForAdditionalForms() if true
        self.offsetForm = False
        self.linearDecayForm_X = False
        self.linearDecayForm_Y = False
        self.linearDecayForm_XY = False
        self.linearDecayAndOffsetForm_X = False
        self.linearDecayAndOffsetForm_Y = False
        self.linearDecayAndOffsetForm_XY = False
        self.linearGrowthForm_X = False
        self.linearGrowthForm_Y = False
        self.linearGrowthForm_XY = False
        self.linearGrowthAndOffsetForm_X = False
        self.linearGrowthAndOffsetForm_Y = False
        self.linearGrowthAndOffsetForm_XY = False
        self.exponentialDecayForm_X = False
        self.exponentialDecayForm_Y = False
        self.exponentialDecayForm_XY = False
        self.exponentialDecayAndOffsetForm_X = False
        self.exponentialDecayAndOffsetForm_Y = False
        self.exponentialDecayAndOffsetForm_XY = False
        self.exponentialGrowthForm_X = False
        self.exponentialGrowthForm_Y = False
        self.exponentialGrowthForm_XY = False
        self.exponentialGrowthAndOffsetForm_X = False
        self.exponentialGrowthAndOffsetForm_Y = False
        self.exponentialGrowthAndOffsetForm_XY = False


    def SetGeneticAlgorithmParameters(self):
        # make sure we are not over maximum possible number of points
        # FindOrCreateReducedDataCache() will usually add one to this to be safe
        self.numberOfDecimatedRawDataPoints = 3 * len(self.coefficientDesignatorTuple) * self.dimensionality # seems to work OK, each equation can override
        if self.numberOfDecimatedRawDataPoints > len(self.DependentDataArray):
            self.numberOfDecimatedRawDataPoints = len(self.DependentDataArray)

        #TODO: change these maximums when more CPU is available
        self.popSize = 100 * len(self.coefficientDesignatorTuple) * self.dimensionality # seems to work OK, each equation can override
        if self.popSize > 500:
            self.popSize = 500

        #TODO: change these maximums when more CPU is available
        self.maxGenerations = 250 * len(self.coefficientDesignatorTuple) * self.dimensionality # seems to work OK, each equation can override
        if self.maxGenerations > 5000:
            self.maxGenerations = 5000


    def ConvertTextToData(self, dataText):

        # you should first process commas before calling this method,
        # as it uses the default token delimiters in string.split()
        #
        # For example, convert $1,234.56 to 1234.56 or 1,23 to 1.23
        # Different number systems have commas in different places
        # and the Python built-in float() method uses decimal notation
        # or scientific notation only

        # cache some data set characteristics for later use,
        # these are for the data domains of individual equations
        self.IndependentData1ContainsZeroFlag = False
        self.IndependentData2ContainsZeroFlag = False
        self.IndependentData1ContainsPositiveFlag = False
        self.IndependentData2ContainsPositiveFlag = False
        self.IndependentData1ContainsNegativeFlag = False
        self.IndependentData2ContainsNegativeFlag = False

        # used in calculation of relative error to prevent divide-by-zero exceptions
        self.DependentDataContainsZeroFlag = False

        # StringIO.StringIO() allows using file methods on text
        rawData = StringIO.StringIO(dataText).readlines()

        # OK, now load in the data
        dataLists = [[], [], []]
        for line in rawData:

            # split the line into string tokens using the default string.split() delimiters
            tokenlist = string.split(line)

            # test this line for minimum required number of string tokens
            if len(tokenlist) < self.dimensionality:
                continue

            # use the python built-in float() conversion and discard if any exceptions
            if self.dimensionality == 1:
                try:
                    a = float(tokenlist[0])
                except:
                    continue
                if a < 0.0:
                    self.IndependentData1ContainsNegativeFlag = True
                elif a > 0.0:
                    self.IndependentData1ContainsPositiveFlag = True
                else:
                    self.IndependentData1ContainsZeroFlag = True

                dataLists[0].append(a)

            if self.dimensionality == 2:
                try:
                    a = float(tokenlist[0])
                    b = float(tokenlist[1])
                except:
                    continue
                if b == 0.0:
                    self.DependentDataContainsZeroFlag = True
                if a < 0.0:
                    self.IndependentData1ContainsNegativeFlag = True
                elif a > 0.0:
                    self.IndependentData1ContainsPositiveFlag = True
                else:
                    self.IndependentData1ContainsZeroFlag = True

                dataLists[0].append(a)
                dataLists[1].append(b)

            if self.dimensionality == 3:
                try:
                    a = float(tokenlist[0])
                    b = float(tokenlist[1])
                    c = float(tokenlist[2])
                except:
                    continue
                if c == 0.0:
                    self.DependentDataContainsZeroFlag = True
                if a < 0.0:
                    self.IndependentData1ContainsNegativeFlag = True
                elif a > 0.0:
                    self.IndependentData1ContainsPositiveFlag = True
                else:
                    self.IndependentData1ContainsZeroFlag = True
                if b < 0.0:
                    self.IndependentData2ContainsNegativeFlag = True
                elif b > 0.0:
                    self.IndependentData2ContainsPositiveFlag = True
                else:
                    self.IndependentData2ContainsZeroFlag = True

                dataLists[0].append(a)
                dataLists[1].append(b)
                dataLists[2].append(c)

        if self.dimensionality == 1:
            self.IndependentDataArray = numpy.array([dataLists[0]])
        if self.dimensionality == 2:
            unused = [1.0] * len(dataLists[0])
            self.IndependentDataArray = numpy.array([dataLists[0], unused]) # the second  _unused_  list is for a bug in scipy.odr, which is used to calculate standard errors on parameters
            self.DependentDataArray = numpy.array(dataLists[1])
        if self.dimensionality == 3:
            self.IndependentDataArray = numpy.array([dataLists[0], dataLists[1]])
            self.DependentDataArray = numpy.array(dataLists[2])


    def CalculatePredictedValuesUsingPassedInCoefficientsAndData(self,  B,  x):

        #any fixed coefficients?
        if self.fixedCoefficientArray != []:
            for i in range(len(self.fixedCoefficientArray)):
                if self.fixedCoefficientArray[i] > -1.0E300:
                    B[i] = self.fixedCoefficientArray[i]
                    
        tempCacheDict = self.cacheDict
        tempDataArray = self.IndependentDataArray
        
        self.cacheDict = {}
        self.IndependentDataArray = x
        self.FindOrCreateCache()
        predictedArray = self.EvaluateCachedData(B, self.cache)
        
        self.cacheDict = tempCacheDict
        self.IndependentDataArray = tempDataArray
        
        return predictedArray


    def CalculateParameterStatistics(self):

        # ensure integers are promoted to floating point with "1.0 * var"
        self.nobs = 1.0 * len(self.DependentDataArray)  # number of observations
        self.ncoef = 1.0 * len(self.coefficientArray)          # number of coef.
        self.df_e = self.nobs - self.ncoef                 # degrees of freedom, error
        self.df_r = self.ncoef - 1.0                              # degrees of freedom, regression

        try:
            self.r2 = 1.0 - self.AbsoluteErrorArray.var()/self.DependentDataArray.var()
        except:
            self.r2 = None

        try:
            self.rmse = numpy.sqrt(numpy.sum(self.AbsoluteErrorArray * self.AbsoluteErrorArray) / self.nobs)
        except:
            self.rmse = None

        try:
            self.r2adj = 1.0 - (1.0 - self.r2)*((self.nobs - 1.0)/(self.nobs-self.ncoef))   # adjusted R-square
        except:
            self.r2adj = None


        try:
            self.Fstat = (self.r2/self.df_r) / ((1.0 - self.r2)/self.df_e)  # model F-statistic
        except:
            self.Fstat = None

        try:
            self.Fpv = 1.0 - scipy.stats.f.cdf(self.Fstat, self.df_r, self.df_e)  # F-statistic p-value
        except:
            self.Fpv = None

        # Model log-likelihood, AIC, and BIC criterion values
        try:
            self.ll = -(self.nobs*0.5)*(1.0 + numpy.log(2.0*numpy.pi)) - (self.nobs*0.5)*numpy.log(numpy.dot(self.AbsoluteErrorArray,self.AbsoluteErrorArray)/self.nobs)
        except:
            self.ll = None

        try:
            self.aic = -2.0*self.ll/self.nobs + (2.0*self.ncoef/self.nobs)
        except:
            self.aic = None

        try:
            self.bic = -2.0*self.ll/self.nobs + (self.ncoef*numpy.log(self.nobs))/self.nobs
        except:
            self.bic = None

        if self.splineFlag != 0: # not appicable to splines
            self.cov_beta = None
            self.sd_beta = None
            self.tstat_beta = None
            self.pstat_beta = None
        else:
            # see both scipy.odr.odrpack and http://www.scipy.org/Cookbook/OLS
            # this is inefficient but works for every possible case
            model = scipy.odr.odrpack.Model(self.CalculatePredictedValuesUsingPassedInCoefficientsAndData)
            realData = scipy.odr.odrpack.RealData(self.IndependentDataArray,  self.DependentDataArray)
            myodr = scipy.odr.odrpack.ODR(realData, model, beta0=self.coefficientArray,  maxit=0)
            myodr.set_job(var_calc=0)
            parameterStatistics = myodr.run()
            self.cov_beta = parameterStatistics.cov_beta # parameter covariance matrix
            self.sd_beta = parameterStatistics.sd_beta    # parameter standard errors

            try:
                self.tstat_beta = self.coefficientArray / self.sd_beta # coeff t-statistics
            except:
                self.tstat_beta = None

            try:
                self.pstat_beta = (1.0 - scipy.stats.t.cdf(numpy.abs(self.tstat_beta), self.df_e)) * 2.0    # coef. p-values
            except:
                self.pstat_beta = None



    def VerifyCoefficientsAreWithinBounds(self, in_coeffArray):
        # have this method return false if outside any desired bounds
        # note that for standard sum-of-squared fitting, the LinearSSQSolverFlag may
        # need to be set to false.  See SetGAParametersAndGuessInitialCoefficientsIfNeeded() for why this is needed.
        return True


    def SetGAParametersAndGuessInitialCoefficientsIfNeeded(self):
        if self.splineFlag != 0: # not appicable to splines
            return
           
        if self.fixedCoefficientArray != []: # user has requested at least one fixed coefficient, must use nonlinear solver
            self.LinearSSQSolverFlag = 0
            
        if self.LinearSSQSolverFlag == 1 and self.fittingTarget == "SSQABS":
            return
            
        self.SetGeneticAlgorithmParameters()
        self.EstimateInitialCoefficients()


    def EvaluateCachedData(self, coeff, indep_data):

        # based on the weave array3d.py example function 'pure_inline'
        code = '''
initializeFunctionPointerArray();

int i;
int* cachedWeaveOffsets = new int[*Nindep_data];
for (i=0; i<*Nindep_data; i++)
    cachedWeaveOffsets[i] = (int)(cachedWeaveOffsetsArray[i]); // longs to ints

// see __init__.py
(*functionArray[equationIndex])(numberOfDataPoints, numberOfCoefficients, cachedWeaveOffsets, coeff, indep_data, resultArray);

delete cachedWeaveOffsets;
'''

        # aliases to make the C++ names match the Python names
        parameterNameList = []

        # these are passed in
        parameterNameList.append('coeff')
        parameterNameList.append('indep_data')

        equationIndex = self.equationIndex
        parameterNameList.append('equationIndex')

        # this offset array is to speed weave's array access, reducing the number of required
        # multiplications in C/C++ loops - in effect partially caching C/C++ data access.
        cachedWeaveOffsetsArray = range(0, len(self.CacheGenerationList) * len(indep_data[0]), len(indep_data[0]))
        parameterNameList.append('cachedWeaveOffsetsArray')

        # quickly make an uninitialized numpy array to hold the results
        resultArray = numpy.empty(len(indep_data[0]))
        parameterNameList.append('resultArray')

        numberOfDataPoints = len(indep_data[0])
        parameterNameList.append('numberOfDataPoints')

        numberOfCoefficients = len(self.coefficientArray)
        parameterNameList.append('numberOfCoefficients')

        scipy.weave.inline(code, parameterNameList, support_code = ExtraCodeForEquationBaseClasses.cppCodeForEvaluate, extra_compile_args = Equation.compiler_flags_for_weave_inline)

        return resultArray


    def CalculateErrors(self):
        self.FindOrCreateCache()
        self.PredictedArray = self.EvaluateCachedData(self.coefficientArray, self.cache)
        self.AbsoluteErrorArray = self.PredictedArray - self.DependentDataArray

        try:
            if self.DependentDataContainsZeroFlag == False:
                self.RelativeErrorArray = self.AbsoluteErrorArray / self.DependentDataArray
                self.PercentErrorArray = self.RelativeErrorArray * 100.0
        except:
            self.DependentDataContainsZeroFlag = True # in effect, yes...
            self.RelativeErrorArray = None
            self.PercentErrorArray = None


    def FindOrCreateCache(self):
        # if the thread's cache dictionary does not exist, create it
        if not self.cacheDict.has_key('AllDataCache'):
            self.cacheDict['AllDataCache'] = {} # create an empty dictionary to fill

        # check each item in the equation's cache generation list
        cacheList = []
        for i in self.CacheGenerationList:
            # if this item is not in the cache, create it and add it to the cache
            if self.cacheDict['AllDataCache'].has_key(i[0]) != 1:
                cacheItem = eval('self.CG_' + i[0].split('_')[0] + "(self.IndependentDataArray, i[1])")
                temp = 'self.CG_' + i[0].split('_')[0] + "(self.IndependentDataArray, i[1])"
                self.cacheDict['AllDataCache'][i[0]] = cacheItem
            cacheList.append(self.cacheDict['AllDataCache'][i[0]])
        self.cache = numpy.array(cacheList)


    def CreateDecimatedRawData(self):
        # create a reduced subset of data, checking along each data dimension
        # to attempt selection of representative data.  This is done by getting indices
        # on the main data set, then drawing from the main data set using those indices

        # if the number of data points in the full data set is not ~1.5 times greater than the desired
        # number of data points in the reduced data set, just use the full data set directly
        if (1.5 * self.numberOfDecimatedRawDataPoints) >= len(self.DependentDataArray):
            self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName] = [self.IndependentDataArray, self.DependentDataArray]
            return

        # find the array indices for the max and min values of each data dimension in the full data set
        minXindex = 0
        maxXindex = 0
        minYindex = 0
        maxYindex = 0
        minZindex = 0
        maxZindex = 0
        for i in range(len(self.DependentDataArray)):
            if self.IndependentDataArray[0][i] < self.IndependentDataArray[0][minXindex]:
                minXindex = i
            if self.IndependentDataArray[0][i] > self.IndependentDataArray[0][maxXindex]:
                maxXindex = i
            if self.dimensionality == 3:
                if self.IndependentDataArray[1][i] < self.IndependentDataArray[1][minYindex]:
                    minYindex = i
                if self.IndependentDataArray[1][i] > self.IndependentDataArray[1][maxYindex]:
                    maxYindex = i
            if self.DependentDataArray[i] < self.DependentDataArray[minZindex]:
                minZindex = i
            if self.DependentDataArray[i] > self.DependentDataArray[maxZindex]:
                maxZindex = i

        # now make an array of the *indices* we will use in the reduced data set.
        # NO duplicate entries desired, so use "if value X not in list Y" logic
        indexList = []
        if minXindex not in indexList:
            indexList += minXindex,
        if maxXindex not in indexList:
            indexList += maxXindex,
        if self.dimensionality == 3:
            if minYindex not in indexList:
                indexList += minYindex,
            if maxYindex not in indexList:
                indexList += maxYindex,
        if minZindex not in indexList:
            indexList += minZindex,
        if maxZindex not in indexList:
            indexList += maxZindex,

        # if we have have not selected all data points, draw more data point indices
        if len(self.DependentDataArray) > len(indexList):
            #create a list of indices sorted by value of dependent variable
            sortedIndexList = range(len(self.DependentDataArray))
            sortedIndexList.sort(self.ReducedDataComparisonFunctionUsedForSorting)

            for i in range(self.numberOfDecimatedRawDataPoints):
                index = i * len(self.DependentDataArray) / self.numberOfDecimatedRawDataPoints
                if index not in indexList:
                    indexList += (index),

        # now that we have all the locations (indices) of the data points in the reduced
        # data set, draw those points from the full data set and make our reduced data cache
        reducedData = [[], [], []]
        for i in indexList:
            reducedData[0].append(self.IndependentDataArray[0][i])
            if self.dimensionality == 3:
                reducedData[1].append(self.IndependentDataArray[1][i])
            reducedData[2].append(self.DependentDataArray[i])

        if self.dimensionality == 2:
            independentDataArray = numpy.array([reducedData[0]])
            dependentDataArray = numpy.array(reducedData[2])
        else:
            independentDataArray = numpy.array([reducedData[0], reducedData[1]])
            dependentDataArray = numpy.array(reducedData[2])

        self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName] = [independentDataArray, dependentDataArray]


    def ReducedDataComparisonFunctionUsedForSorting(self, int1, int2):
        if self.DependentDataArray[int1] < self.DependentDataArray[int2]:
            return -1
        if self.DependentDataArray[int1] > self.DependentDataArray[int2]:
            return 1
        return 0


    def FindOrCreateReducedDataCache(self):
        # these are for looking up values in the Reduced Data cache
        self.decimatedRawDataCacheName = 'DecimatedRawDataCache' + str(self.numberOfDecimatedRawDataPoints) + '_' + str(self.dimensionality) + 'D'
        self.reducedDataName = 'ReducedDataCache' + str(self.numberOfDecimatedRawDataPoints) + '_' + str(self.dimensionality) + 'D'

        # if the Decimated Raw Data cache dictionaries do not exist, create them
        if self.cacheDict.has_key('DecimatedRawDataCache') != 1:
            self.cacheDict['DecimatedRawDataCache'] = {}

        # if the Reduced Data cache dictionaries do not exist, create them
        if self.cacheDict.has_key('ReducedDataCache') != 1:
            self.cacheDict['ReducedDataCache'] = {}

        # if we don't have a reduced data set for [this number of points], first
        # check if there is one for [this number +1].  If not a +1 to use, then make this set
        if self.cacheDict['DecimatedRawDataCache'].has_key(self.decimatedRawDataCacheName) != 1:
            nextSetName = 'DecimatedRawDataCache' + str(self.numberOfDecimatedRawDataPoints + 1) + '_' + str(self.dimensionality) + 'D' # +1 is OK for estimation
            if self.cacheDict['DecimatedRawDataCache'].has_key(nextSetName) != 1:
                self.CreateDecimatedRawData() # create the Decimated Raw Data cache for this number of Reduced Data points
            else:
                self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName] = self.cacheDict['DecimatedRawDataCache'][nextSetName]

        # check each item in the equation's cache generation list
        if self.cacheDict['ReducedDataCache'].has_key(self.reducedDataName) != 1: # if a cache for this number of points does not exist, create it
            self.cacheDict['ReducedDataCache'][self.reducedDataName] = {}
        tempReducedDataCacheList = []
        for i in self.CacheGenerationList:
            # if this item is not in the Reduced Data cache, create it and add it to the cache
            if self.cacheDict['ReducedDataCache'][self.reducedDataName].has_key(i[0]) != 1:
                cacheItem = eval('self.CG_' + i[0].split('_')[0] + "(self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName][0], i[1])"),
                self.cacheDict['ReducedDataCache'][self.reducedDataName][i[0]] = cacheItem
            tempReducedDataCacheList.append(self.cacheDict['ReducedDataCache'][self.reducedDataName][i[0]][0])

        # now store this cache and associated dependent data conveniently for later retrieval
        self.reducedCache = numpy.array(tempReducedDataCacheList)
        self.reducedDependentData = self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName][1]


    def Initialize(self):
        if 0 != self.requiresCoefficientBoundsCheckWhenFitting:
            self.upperCoefficientBoundsArray = numpy.ones(len(self.coefficientDesignatorTuple)) * 1.0E300
            self.lowerCoefficientBoundsArray = numpy.ones(len(self.coefficientDesignatorTuple)) * -1.0E300
        self.CreateCacheGenerationList()


    def CalculateFittingTarget(self, in_coeffArray):
        #save time by checking constraints and bounds first
        if not self.VerifyCoefficientsAreWithinBounds(in_coeffArray):
            return 1.0E300

        try:
            #any fixed coefficients?
            if self.fixedCoefficientArray != []:
                for i in range(len(self.fixedCoeffArray)):
                    if self.fixedCoefficientArray[i] > -1.0E300:
                        in_coeffArray[i] = self.fixedCoefficientArray[i]
                        
            error = self.EvaluateCachedData(in_coeffArray, self.cache) - self.DependentDataArray
            if self.fittingTarget == "SSQABS":
                return numpy.sum(error * error)
            if self.fittingTarget == "SSQREL":
                error = error / self.DependentDataArray
                return numpy.sum(error * error)
            if self.fittingTarget == "ABSABS":
                return numpy.sum(numpy.abs(error))
            if self.fittingTarget == "ABSREL":
                return numpy.sum(numpy.abs(error / self.DependentDataArray))
            if self.fittingTarget == "PEAKABS":
                return numpy.max(numpy.abs(error))
            if self.fittingTarget == "PEAKREL":
                return numpy.max(numpy.abs(error / self.DependentDataArray))

            ncoef = 1.0 * len(in_coeffArray)          # number of coef.
            nobs = 1.0 * len(self.DependentDataArray)  # number of observations
            ll = -(nobs*0.5)*(1.0 + numpy.log(2.0*numpy.pi)) - (nobs*0.5)*numpy.log(numpy.dot(error,error)/nobs)

            if self.fittingTarget == "AIC":
                return -2.0*ll/nobs + (2.0*ncoef/nobs)
            if self.fittingTarget == "BIC":
                return -2.0*ll/nobs + (ncoef*numpy.log(nobs))/nobs
            if self.fittingTarget == "ODR": # this is inefficient but works for every possible case
                model = scipy.odr.odrpack.Model(self.CalculatePredictedValuesUsingPassedInCoefficientsAndData)
                realData = scipy.odr.odrpack.RealData(self.IndependentDataArray,  self.DependentDataArray)
                myodr = scipy.odr.odrpack.ODR(realData, model, beta0=self.coefficientArray,  maxit=len(self.coefficientArray) * self.fminIterationLimit)
                myodr.set_job(var_calc=0)
                out = myodr.run()
                return out.sum_square
        except:
            return 1.0E300


    def CalculateReducedDataFittingTarget(self, in_coeffArray):
        #save time by checking constraints and bounds first
        if not self.VerifyCoefficientsAreWithinBounds(in_coeffArray):
            return 1.0E300

        # just return SSQ, as we are only using this method for guessing initial coefficients
        try:
            #any fixed coefficients?
            if self.fixedCoefficientArray != []:
                for i in range(len(self.fixedCoeffArray)):
                    if self.fixedCoefficientArray[i] > 1.0E300:
                        in_coeffArray[i] = self.fixedCoefficientArray[i]
                        
            error = self.EvaluateCachedData(in_coeffArray, self.reducedCache) - self.reducedDependentData
            ssq = numpy.sum(error * error)
        except:
            ssq = 1.0E300
        return ssq


    def FitToCacheData(self):
        self.FindOrCreateCache()

        # linear fits - this may have been set to zero in SetGAParametersAndGuessInitialCoefficientsIfNeeded()
        # if there are fixed coefficients
        if self.LinearSSQSolverFlag != 0: # can we linear fit?
            if self.fittingTarget == "SSQABS":
                self.coefficientArray = numpy.linalg.lstsq(self.cache.T, self.DependentDataArray)[0]
                return

        #nonlinear fits
        if self.fittingTarget == "ODR":
                model = scipy.odr.odrpack.Model(self.CalculatePredictedValuesUsingPassedInCoefficientsAndData)
                realData = scipy.odr.odrpack.RealData(self.IndependentDataArray,  self.DependentDataArray)
                myodr = scipy.odr.odrpack.ODR(realData, model, beta0=self.coefficientArray,  maxit=len(self.coefficientArray) * self.fminIterationLimit)
                myodr.set_job(fit_type=0, deriv=0) # explicit ODR, faster forward-only finite differences for derivatives
                out = myodr.run()
                self.coefficientArray = out.beta
        else:
            if self.nonlinearSolver == 1: # 1 = fmin, 2 = fmin_bfgs, 3 = fmin_cg, 4 = fmin_ncq, 5 = fmin_powell
                self.coefficientArray = scipy.optimize.fmin(self.CalculateFittingTarget, self.coefficientArray, maxiter = len(self.coefficientArray) * self.fminIterationLimit, maxfun = len(self.coefficientArray) * self.fminFunctionLimit, disp = 0, xtol=self.fmin_xtol, ftol=self.fmin_ftol)
            elif self.nonlinearSolver == 2:
                self.coefficientArray = scipy.optimize.fmin_bfgs(self.CalculateFittingTarget, self.coefficientArray, maxiter = len(self.coefficientArray) * self.fminIterationLimit, disp = 0)
            elif self.nonlinearSolver == 3:
                self.coefficientArray = scipy.optimize.fmin_cg(self.CalculateFittingTarget, self.coefficientArray, maxiter = len(self.coefficientArray) * self.fminIterationLimit, disp = 0)
            elif self.nonlinearSolver == 4:
                self.coefficientArray = scipy.optimize.fmin_ncg(self.CalculateFittingTarget, self.coefficientArray, maxiter = len(self.coefficientArray) * self.fminIterationLimit, disp = 0)
            elif self.nonlinearSolver == 5:
                self.coefficientArray = scipy.optimize.fmin_powell(self.CalculateFittingTarget, self.coefficientArray, maxiter = len(self.coefficientArray) * self.fminIterationLimit, disp = 0, xtol=1e-10, ftol=1e-10)

            #any fixed coefficients?
            if self.fixedCoefficientArray != []:
                for i in range(len(self.fixedCoefficientArray)):
                    if self.fixedCoefficientArray[i] > -1.0E300:
                        self.coefficientArray[i] = self.fixedCoefficientArray[i]


    def EstimateInitialCoefficients(self):
        # Values in the existing coefficientArray will be placed into the initial DE population
        self.FindOrCreateReducedDataCache()

        if len(self.coefficientDesignatorTuple) != len(self.coefficientArray):
            self.coefficientArray = numpy.ones(len(self.coefficientDesignatorTuple))

        # based on the weave array3d.py example function 'pure_inline'
        supportCode = '''
// adapted for scipy.weave by James Phillips http://zunzun.com April 2008
// from Differential Evolution Program: de.c  Version: 3.6
// DE homepage: http://www.icsi.berkeley.edu/~storn/code.html

/*------Constants for rnd_uni()--------------------------------------------*/
#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


void  assignd(int D, double *a, double *b)
// Assigns D-dimensional vector b to vector a.
{
	for (int j=0; j<D; j++)
	a[j] = b[j];
}

double rnd_uni(long *idum)
// generates an equally distributed random number in the interval [0,1]
{
	long j;
	long k;
	static long idum2=123456789;
	static long iy=0;
	static long iv[NTAB];
	double temp;

	if (*idum <= 0)
	{
		if (-(*idum) < 1) *idum=1;
		else *idum = -(*idum);
		idum2=(*idum);
		for (j=NTAB+7;j>=0;j--)
		{
			k=(*idum)/IQ1;
			*idum=IA1*(*idum-k*IQ1)-k*IR1;
			if (*idum < 0) *idum += IM1;
			if (j < NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ1;
	*idum=IA1*(*idum-k*IQ1)-k*IR1;
	if (*idum < 0) *idum += IM1;
	k=idum2/IQ2;
	idum2=IA2*(idum2-k*IQ2)-k*IR2;
	if (idum2 < 0) idum2 += IM2;
	j=iy/NDIV;
	iy=iv[j]-idum2;
	iv[j] = *idum;
	if (iy < 1) iy += IMM1;
	if ((temp=AM*iy) > RNMX) return RNMX;
	else return temp;
}

''' + ExtraCodeForEquationBaseClasses.cppCodeForGA


        # aliases to make the C++ names match the Python names
        parameterNameList = []

        deltaZ = self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName][1].max() - self.cacheDict['DecimatedRawDataCache'][self.decimatedRawDataCacheName][1].min()

        sufficientSolution = deltaZ / self.guessDivisor
        parameterNameList.append('sufficientSolution')

        # this offset array is to speed weave's array access, reducing the number of required
        # multiplications in C/C++ loops - in effect partially caching C/C++ data access.
        cachedWeaveOffsetsArray = numpy.array(range(0, len(self.CacheGenerationList) * len(self.reducedCache[0]), len(self.reducedCache[0])))
        parameterNameList.append('cachedWeaveOffsetsArray')

        equationIndex = self.equationIndex
        parameterNameList.append('equationIndex')

        indep_data = self.reducedCache
        parameterNameList.append('indep_data')

        dep_data = self.reducedDependentData
        parameterNameList.append('dep_data')

        numberOfDataPoints = len(self.reducedDependentData)
        parameterNameList.append('numberOfDataPoints')

        inibound_h = self.maxparamvalue
        parameterNameList.append('inibound_h')

        inibound_l = self.minparamvalue
        parameterNameList.append('inibound_l')

        genmax = self.maxGenerations
        parameterNameList.append('genmax')

        F = self.diffScale
        parameterNameList.append('F')

        CR = self.crossoverProb
        parameterNameList.append('CR')

        populationSize = self.popSize
        parameterNameList.append('populationSize')

        coeffs = self.coefficientArray
        parameterNameList.append('coeffs')

        numberOfCoefficients = len(self.coefficientArray)
        parameterNameList.append('numberOfCoefficients')

        useFixedCoefficientArrayFlag = 0
        if self.fixedCoefficientArray != []:
            useFixedCoefficientArrayFlag = 1
        parameterNameList.append('useFixedCoefficientArrayFlag')

        fixedCoefficientArray = self.fixedCoefficientArray
        parameterNameList.append('fixedCoefficientArray')

        shouldBoundsBeChecked = self.requiresCoefficientBoundsCheckWhenFitting
        if 0 != self.requiresCoefficientBoundsCheckWhenFitting:
            upperConstraints = self.upperCoefficientBoundsArray
            lowerConstraints = self.lowerCoefficientBoundsArray
        else:
            upperConstraints = self.reducedDependentData # unused, give something that stops compiler warnings
            lowerConstraints = self.reducedDependentData # unused, give something that stops compiler warnings
        parameterNameList.append('shouldBoundsBeChecked')
        parameterNameList.append('upperConstraints')
        parameterNameList.append('lowerConstraints')

        seed = 3
        parameterNameList.append('seed')

        target = 1
        parameterNameList.append('target')

        code = '''
    initializeFunctionPointerArray();

    // adapted for scipy.weave by James Phillips http://zunzun.com April 2008
    // from Differential Evolution Program: de.c  Version: 3.6
    // DE homepage: http://www.icsi.berkeley.edu/~storn/code.html

    int D = numberOfCoefficients;

    long  rnd_uni_init;    // serves as a seed for rnd_uni()

    int   i, j, L, n;      // counting variables
    int   r1, r2, r3, r4;  // placeholders for random indexes
    int   r5;              // placeholders for random indexes
    int   imin;            // index to member with lowest energy
    int   gen;

    double trial_cost;      // buffer variable
    double *tmp, *best;     // internal members
    double *bestit;         // internal members
    double *cost;           // obj. funct. values
    double **c, **d;        // internal members
    double **pold, **pnew;  // internal members
    double **pswap;         // internal members
    double cmin;            // help variables

    int ndatapoints = numberOfDataPoints;

    /*-----Initialize random number generator-----------------------------*/

    rnd_uni_init = -(long)seed;  /* initialization of rnd_uni() */

    int* cachedWeaveOffsets = new int[*Nindep_data];
    for (i=0; i<*Nindep_data; i++)
        cachedWeaveOffsets[i] = (int)(cachedWeaveOffsetsArray[i]); // longs to ints

    c = new double* [populationSize];
    for (i=0; i<populationSize; i++)
        c[i] = new double[D];

    d = new double* [populationSize];
    for (i=0; i<populationSize; i++)
        d[i] = new double[D];

    tmp = new double[D];
    best = new double[D];
    bestit = new double[D];
    cost = new double[populationSize];

    // random initial population
    for (i=0; i<populationSize; i++)
        for (j=0; j<D; j++)
            c[i][j] = inibound_l + rnd_uni(&rnd_uni_init)*(inibound_h - inibound_l);

    // pre-seed the first generation's list with passed-in parameter values
    for (j=0; j<D; j++)
        c[0][j] = coeffs[j];

    double * coeff;

    for (j=0; j<populationSize; j++)
    {
        coeff = c[j];
        cost[j] = 0.0;
        int boundsTestOK = 1;

        // check for parameter constraints
        if(0 != shouldBoundsBeChecked)
        {
            for(i=0; i<numberOfCoefficients; i++)
                if ((coeff[i] < lowerConstraints[i]) || (coeff[i]  > upperConstraints[i]))
                {
                    cost[j] = 1.0E300;
                    boundsTestOK = 0;
                }
        }

        if(0 != boundsTestOK)
        {
            // see __init__.py
            cost[j] = (*functionArray[equationIndex])(ndatapoints, numberOfCoefficients, cachedWeaveOffsets, coeff, indep_data, dep_data);
        }
    }

    cmin = cost[0];
    imin = 0;
    for (i=1; i<populationSize; i++)
    {
        if (cost[i]<cmin)
        {
            cmin = cost[i];
            imin = i;
        }
    }

    assignd(D,best,c[imin]);            /* save best member ever          */

    assignd(D,bestit,c[imin]);          /* save best member of generation */

    pold = c; /* old population (generation G)   */
    pnew = d; /* new population (generation G+1) */


    /*=======================================================================*/
    /*=========Iteration loop================================================*/
    /*=======================================================================*/

    gen = 0;                          /* generation counter reset */

    while ((gen < genmax) && (sufficientSolution < cmin))
    {
        for (j=0; j<populationSize; j++)         /* Start of loop through ensemble  */
        {
            do                        /* Pick a random population member */
            {                         /* endless loop for populationSize < 2 !!!     */
                r1 = (int)(rnd_uni(&rnd_uni_init)*populationSize);
            }while(r1==j);

            do                        /* Pick a random population member */
            {                         /* endless loop for populationSize < 3 !!!     */
                r2 = (int)(rnd_uni(&rnd_uni_init)*populationSize);
            }while((r2==j) || (r2==r1));

            do                        /* Pick a random population member */
            {                         /* endless loop for populationSize < 4 !!!     */
                r3 = (int)(rnd_uni(&rnd_uni_init)*populationSize);
            }while((r3==j) || (r3==r1) || (r3==r2));

            do                        /* Pick a random population member */
            {                         /* endless loop for populationSize < 5 !!!     */
                r4 = (int)(rnd_uni(&rnd_uni_init)*populationSize);
            }while((r4==j) || (r4==r1) || (r4==r2) || (r4==r3));

            do                        /* Pick a random population member */
            {                         /* endless loop for populationSize < 6 !!!     */
                r5 = (int)(rnd_uni(&rnd_uni_init)*populationSize);
            }while((r5==j) || (r5==r1) || (r5==r2) || (r5==r3) || (r5==r4));

            /*-------DE/best/1/bin--------------------------------------------------------------------*/
            assignd(D,tmp,pold[j]);
            n = (int)(rnd_uni(&rnd_uni_init)*D);
            for (L=0; L<D; L++) /* perform D binomial trials */
            {
                if ((rnd_uni(&rnd_uni_init) < CR) || L == (D-1)) /* change at least one parameter */
                {
                    tmp[n] = bestit[n] + F*(pold[r2][n]-pold[r3][n]);
                }
                n = (n+1)%D;
            }


            /*=======Trial mutation now in tmp[]. Test how good this choice really was.==================*/

            double * coeff;
            coeff = tmp;
            double temp, temp_x_sq, temp_y_sq, s_sq, s_over_r, s_sq_b, s_over_r_b;

            trial_cost = 0.0;

            int boundsTestOK = 1;

            // check for parameter constraints
            if(0 != shouldBoundsBeChecked)
            {
                for(i=0; i<numberOfCoefficients; i++)
                {
                    if ((coeff[i] < lowerConstraints[i]) || (coeff[i]  > upperConstraints[i]))
                    {
                        trial_cost = 1.0E300;
                        boundsTestOK = 0;
                    }
                }
            }

            if(0 != boundsTestOK)
            {
                if(0 != useFixedCoefficientArrayFlag) // for performance, use flag to enter this code section
                {
                    for (int fixedIndex = 0; fixedIndex < numberOfCoefficients; fixedIndex++)
                    {
                        if (fixedCoefficientArray[fixedIndex] > -1.0E300)
                        {
                            coeff[fixedIndex] = fixedCoefficientArray[fixedIndex];
                        }
                    }
                }
                
                // see __init__.py
                trial_cost = (*functionArray[equationIndex])(ndatapoints, numberOfCoefficients, cachedWeaveOffsets, coeff, indep_data, dep_data);
            }

            if (trial_cost <= cost[j])   /* improved objective function value ? */
            {
                cost[j]=trial_cost;
                assignd(D,pnew[j],tmp);
                if (trial_cost<cmin)          /* Was this a new minimum? */
                {                               /* if so...*/
                    cmin=trial_cost;           /* reset cmin to new low...*/
                    imin=j;
                    assignd(D,best,tmp);
                }
            }
            else
            {
                assignd(D,pnew[j],pold[j]); /* replace target with old value */
            }
        }   /* End mutation loop through pop. */

            assignd(D,bestit,best);  /* Save best population member of current iteration */

            /* swap population arrays. New generation becomes old one */

            pswap = pold;
            pold  = pnew;
            pnew  = pswap;

            gen++;

        /*=======================================================================*/
        /*=========End of iteration loop=========================================*/
        /*=======================================================================*/
    }

    for(i=0; i<D; i++)
        coeffs[i] = best[i];

    delete cost;
    delete bestit;
    delete best;
    delete tmp;

    for (i=0; i<populationSize; i++)
        delete d[i];
    delete d;
    for (i=0; i<populationSize; i++)
        delete c[i];
    delete c;
    delete cachedWeaveOffsets;
'''


        scipy.weave.inline(code, parameterNameList, support_code = supportCode, extra_compile_args = Equation.compiler_flags_for_weave_inline)


    # this method is also overridden
    def GetAdditionalDesignator(self, n):
        return Equation.ascii[n]

    def RegenerateForm(self):
        pass # mosty for polyXials


    def CodePYTHON(self):
        s  = """# To the best of my knowledge this code is correct.
# If you find any errors or problems please contact
# me at zunzun@zunzun.com.
#      James


import math

"""
        s += "# " + self.FittingTargetDict[self.fittingTarget][1] + "\n\n"
        if self.dimensionality == 2:
            s += "def " + self.__class__.__name__ + "_model(x_in):\n"
        else:
            s += "def " + self.__class__.__name__ + "_model(x_in, y_in):\n"
        s += "\ttemp = 0.0\n\n"

        s += "\t# coefficients\n"
        for i in range(len(self.coefficientArray)):
            s += "\t" + self.coefficientDesignatorTuple[i] + " = %-.16E" % (self.coefficientArray[i]) + "\n"
        s += "\n"

        s += self.SpecificCodePYTHON()

        s += "\treturn temp\n"
        return s


    def CodeCPP(self):
        s  = """// To the best of my knowledge this code is correct.
// If you find any errors or problems please contact
// me at zunzun@zunzun.com.
//      James


#include <math.h>

"""
        s += "// " + self.FittingTargetDict[self.fittingTarget][1] + "\n\n"
        if self.dimensionality == 2:
            s += "double " + self.__class__.__name__ + "_model(double x_in)\n{\n"
        else:
            s += "double " + self.__class__.__name__ + "_model(double x_in, double y_in)\n{\n"
        s += "\tdouble temp;\n\ttemp = 0.0;\n\n"

        s += "\t// coefficients\n"
        for i in range(len(self.coefficientArray)):
            s += "\tdouble " + self.coefficientDesignatorTuple[i] + " = %-.16E" % (self.coefficientArray[i]) + ";\n"
        s += "\n"
        s += self.SpecificCodeCPP()
        s += "\treturn temp;\n}\n"
        return s


    def CodeJAVA(self):
        s  = """// To the best of my knowledge this code is correct.
// If you find any errors or problems please contact
// me at zunzun@zunzun.com.
//      James


import java.lang.Math;

"""
        s += "// " + self.FittingTargetDict[self.fittingTarget][1] + "\n\n"
        s += "class " + self.__class__.__name__ + "\n{\n"
        if self.dimensionality == 2:
            s += "\tdouble " + self.__class__.__name__ + "_model(double x_in)\n\t{\n"
        else:
            s += "\tdouble " + self.__class__.__name__ + "_model(double x_in, double y_in)\n\t{\n"
        s += "\t\tdouble temp;\n\t\ttemp = 0.0;\n\n"

        s += "\t\t// coefficients\n"
        for i in range(len(self.coefficientArray)):
            s += "\t\tdouble " + self.coefficientDesignatorTuple[i] + " = %-.16E" % (self.coefficientArray[i]) + ";\n"
        s += "\n"
        s += self.SpecificCodeJAVA()
        s += "\t\treturn temp;\n\t}\n}\n"
        return s


    def CodeCS(self):
        s  = """// To the best of my knowledge this code is correct.
// If you find any errors or problems please contact
// me at zunzun@zunzun.com.
//      James


using System;

"""
        s += "// " + self.FittingTargetDict[self.fittingTarget][1] + "\n\n"
        s += "class " + self.__class__.__name__ + "\n{\n"
        if self.dimensionality == 2:
            s += "\tdouble " + self.__class__.__name__ + "_model(double x_in)\n\t{\n"
        else:
            s += "\tdouble " + self.__class__.__name__ + "_model(double x_in, double y_in)\n\t{\n"
        s += "\t\tdouble temp;\n\t\ttemp = 0.0;\n\n"

        s += "\t\t// coefficients\n"
        for i in range(len(self.coefficientArray)):
            s += "\t\tdouble " + self.coefficientDesignatorTuple[i] + " = %-.16E" % (self.coefficientArray[i]) + ";\n"
        s += "\n"
        s += self.SpecificCodeCS()
        s += "\t\treturn temp;\n\t}\n}\n"
        return s


    def CodeSCILAB(self):
        s  = """// To the best of my knowledge this code is correct.
// If you find any errors or problems please contact
// me at zunzun@zunzun.com.
//      James

"""
        s += "// " + self.FittingTargetDict[self.fittingTarget][1] + "\n\n"
        if self.dimensionality == 2:
            s += "function y=" + self.__class__.__name__ + "_model(x_in)\n"
        else:
            s += "function z=" + self.__class__.__name__ + "_model(x_in, y_in)\n"
        s += "\ttemp = 0.0\n\n"

        s += "\t// coefficients\n"
        for i in range(len(self.coefficientArray)):
            s += "\t" + self.coefficientDesignatorTuple[i] + " = %-.16E" % (self.coefficientArray[i]) + "\n"
        s += "\n"
        s += self.SpecificCodeSCILAB()
        if self.dimensionality == 2:
            s += "\n\ty = temp\n"
        else:
            s += "\n\tz = temp\n"
        s += "endfunction\n"
        return s


    def CodeMATLAB(self):
        s  = """% To the best of my knowledge this code is correct.
% If you find any errors or problems please contact
% me at zunzun@zunzun.com.
%      James

"""
        s += "% " + self.FittingTargetDict[self.fittingTarget][1] + "\n\n"
        if self.dimensionality == 2:
            s += "function y=" + self.__class__.__name__ + "_model(x_in)\n"
        else:
            s += "function z=" + self.__class__.__name__ + "_model(x_in, y_in)\n"
        s += "\ttemp = 0.0;\n\n"

        s += "\t% coefficients\n"
        for i in range(len(self.coefficientArray)):
            s += "\t" + self.coefficientDesignatorTuple[i] + " = %-.16E" % (self.coefficientArray[i]) + ";\n"
        s += "\n"

        # semicolons on lines internal to functions
        #  comment designators
        #  replace "^" with ".^"
        s += self.SpecificCodeSCILAB().replace('\n', ';\n').replace('//', '%') .replace('^', '.^') .replace('*', '.*').replace('/', './')

        if self.dimensionality == 2:
            s += "\n\ty = temp;\n"
        else:
            s += "\n\tz = temp;\n"
        return s




class Equation2D(Equation):

    def __init__(self):
        Equation.__init__(self)
        self.dimensionality = 2
        self.exampleData = """
 5.357    0.376
 5.457    0.489
 5.797    0.874
 5.936    1.049
 6.161    1.327
 6.697    2.054
 6.731    2.077
 6.775    2.138
 8.442    4.744
 9.769    7.068
 9.861    7.104
 """

    def CommonInitCodeForAdditionalForms(self, numberOfAdditionalCoefficients):
        self.__class__.__bases__[0].__init__(self)
        self.extendedForm = True # extended form of the equation?
        self.previousNumberOfCoefficients = len(self.coefficientDesignatorTuple)
        clist = list(self.coefficientDesignatorTuple)
        self.additionalDesignatorList = []
        for i in range(numberOfAdditionalCoefficients):
            self.additionalDesignatorList.append(self.GetAdditionalDesignator(self.previousNumberOfCoefficients + i))
            clist.append(self.GetAdditionalDesignator(self.previousNumberOfCoefficients + i))
        self.coefficientDesignatorTuple = tuple(clist)


    def InitCodeForOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.offsetForm = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList___Offset, self, self.__class__)
        self.function_cpp_code = self.function_cpp_code.replace(';// ', ' + coeff[' + str(self.previousNumberOfCoefficients) + '];// ')
        self.name += " With Offset"
        self.HTML += " + " + self.additionalDesignatorList[0]
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP___Offset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA___Offset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS___Offset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON___Offset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB___Offset, self, self.__class__)


    def InitCodeFor_X_LinearDecayFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.linearDecayForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_LinearDecay, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        if self.dimensionality == 2:
            self.name += " With Linear Decay"
        else:
            self.name += " With X Linear Decay"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * x)"
        self.RequiresNonzeroX = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_LinearDecay, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_LinearDecay, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_LinearDecay, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_LinearDecay, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_LinearDecay, self, self.__class__)


    def InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.linearDecayAndOffsetForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_LinearDecayAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        if self.dimensionality == 2:
            self.name += " With Linear Decay And Offset"
        else:
            self.name += " With X Linear Decay And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * x) + " + self.additionalDesignatorList[1]
        self.RequiresNonzeroX = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_LinearDecayAndOffset, self, self.__class__)


    def InitCodeFor_X_ExponentialDecayFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.exponentialDecayForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_ExponentialDecay, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        if self.dimensionality == 2:
            self.name += " With Exponential Decay"
        else:
            self.name += " With X Exponential Decay"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * e<SUP>x</SUP>)"
        self.RequiresNonzeroX = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_ExponentialDecay, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_ExponentialDecay, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_ExponentialDecay, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_ExponentialDecay, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_ExponentialDecay, self, self.__class__)


    def InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.exponentialDecayAndOffsetForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_ExponentialDecayAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        if self.dimensionality == 2:
            self.name += " With Exponential Decay And Offset"
        else:
            self.name += " With X Exponential Decay And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * e<SUP>x</SUP>) + " + self.additionalDesignatorList[1]
        self.RequiresNonzeroX = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_ExponentialDecayAndOffset, self, self.__class__)


    def InitCodeFor_X_LinearGrowthFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.linearGrowthForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_LinearGrowth, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        if self.dimensionality == 2:
            self.name += " With Linear Growth"
        else:
            self.name += " With X Linear Growth"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * x)"
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_LinearGrowth, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_LinearGrowth, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_LinearGrowth, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_LinearGrowth, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_LinearGrowth, self, self.__class__)


    def InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.linearGrowthAndOffsetForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_LinearGrowthAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        if self.dimensionality == 2:
            self.name += " With Linear Growth And Offset"
        else:
            self.name += " With X Linear Growth And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * x) + " + self.additionalDesignatorList[1]
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_LinearGrowthAndOffset, self, self.__class__)


    def InitCodeFor_X_ExponentialGrowthFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.exponentialGrowthForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_ExponentialGrowth, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        if self.dimensionality == 2:
            self.name += " With Exponential Growth"
        else:
            self.name += " With X Exponential Growth"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * e<SUP>x</SUP>)"
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_ExponentialGrowth, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_ExponentialGrowth, self, self.__class__)


    def InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.exponentialGrowthAndOffsetForm_X = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__X_ExponentialGrowthAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        if self.dimensionality == 2:
            self.name += " With Exponential Growth And Offset"
        else:
            self.name += " With X Exponential Growth And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * e<SUP>x</SUP>) + " + self.additionalDesignatorList[1]
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__X_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__X_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__X_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__X_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__X_ExponentialGrowthAndOffset, self, self.__class__)


    def CG_Ones(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'Ones'
        return numpy.ones_like(data[0])

    def CG_X(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'X'
        return data[0]

    def CG_NegX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'NegX'
        return -1.0 * data[0]

    def CG_SinX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'SinX'
        try:
            return numpy.sin(data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_TanX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'TanX'
        try:
            return numpy.tan(data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_CoshX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'CoshX'
        try:
            return numpy.cosh(data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_LogX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'LogX'
        try:
            return numpy.log(data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_Log10X(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'Log10X'
        try:
            return numpy.log10(data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_InverseX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'InverseX'
        try:
            return 1.0 / data[0]
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_PowX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'PowX_' + str(args[0])
        try:
            return numpy.power(data[0], args[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_PowLogX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'PowLogX_' + str(args[0])
        try:
            return numpy.power(numpy.log(data[0]), args[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_Polyfunctional2D(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'Polyfunctional2D_' + str(args)
        return self.Polyfun2DEqList[args].value(data[0])

    def CG_TwoPiX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'TwoPiX'
        try:
            return 2.0 * math.pi * data[0]
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_NistEnsoCosX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'NistEnsoCosX'
        try:
            return numpy.cos(2.0 * math.pi * data[0] / 12.0)
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_NistEnsoSinX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'NistEnsoSinX'
        try:
            return numpy.sin(2.0 * math.pi * data[0] / 12.0)
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_ExpX(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'ExpX'
        try:
            return numpy.exp(data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])


class Equation3D(Equation2D):

    def __init__(self):
        Equation.__init__(self)
        self.dimensionality = 3
        self.exampleData = """
 3.017  2.175   0.320
 2.822  2.624   0.629
 2.632  2.839   0.950
 2.287  3.030   1.574
 2.207  3.057   1.725
 2.048  3.098   2.035
 1.963  3.115   2.204
 1.784  3.144   2.570
 1.712  3.153   2.721
 2.972  2.106   0.313
 2.719  2.542   0.643
 2.495  2.721   0.956
 2.070  2.878   1.597
 1.969  2.899   1.758
 1.768  2.929   2.088
 1.677  2.939   2.240
 1.479  2.957   2.583
 1.387  2.963   2.744
 2.843  1.984   0.315
 2.485  2.320   0.639
 2.163  2.444   0.954
 1.687  2.525   1.459
 1.408  2.547   1.775
 1.279  2.554   1.927
 1.016  2.564   2.243
 0.742  2.568   2.581
 0.607  2.571   2.753
 """
    def CG_ExpXY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'ExpXY'
        try:
            return numpy.exp(data[0] * data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_XSQMINUSYSQ(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'XSQMINUSYSQ'
        try:
            return (data[0] * data[0]) - (data[1] * data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_YSQMINUSXSQ(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'YSQMINUSXSQ'
        try:
            return (data[1] * data[1]) - (data[0] * data[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_XSQPLUSYSQ(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'XSQPLUSYSQ'
        try:
            return (data[0] * data[0]) + (data[1] * data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_ExpY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'ExpY'
        try:
            return numpy.exp(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_Polyfunctional3D(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'Polyfunctional3D_' + str(args)
        return self.Polyfun3DEqList[args[0]].value(data[0]) * self.Polyfun3DEqList[args[1]].value(data[1])

    def CG_PowLogY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'PowLogY_' + str(args[0])
        try:
            return numpy.power(numpy.log(data[1]), args[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_PowXY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'PowXY_' + str(args[0]) + str(args[1])
        try:
            return numpy.power(data[0], args[0]) * numpy.power(data[1], args[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_PowLogXLogY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'PowLogXLogY_' + str(args[0]) + str(args[1])
        try:
            return numpy.power(numpy.log(data[0]), args[0]) * numpy.power(numpy.log(data[1]), args[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_PowY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'PowY_' + str(args[0])
        try:
            return numpy.power(data[1], args[0])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_InverseY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'InverseY'
        try:
            return 1.0 / data[1]
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_Log10Y(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'Log10Y'
        try:
            return numpy.log10(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_LogY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'LogY'
        try:
            return numpy.log(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_LogXY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'LogXY'
        try:
            return numpy.log(data[0] * data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_CoshY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'CoshY'
        try:
            return numpy.cosh(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_TanY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'TanY'
        try:
            return numpy.tan(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_SinY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'SinY'
        try:
            return numpy.sin(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_NegY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'NegY'
        return -1.0 * data[1]

    def CG_Y(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'Y'
        return data[1]

    def CG_XY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'XY'
        try:
            return data[0] * data[1]
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def CG_LogXLogY(self, data=None, args=None, NameOrValueFlag=0):
        if NameOrValueFlag == 1:
            return 'LogXLogY'
        try:
            return numpy.log(data[0]) * numpy.log(data[1])
        except:
            return 1.0E300 * numpy.ones_like(data[0])

    def InitCodeFor_Y_LinearDecayFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.linearDecayForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_LinearDecay, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With Y Linear Decay"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * y)"
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_LinearDecay, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_LinearDecay, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_LinearDecay, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_LinearDecay, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_LinearDecay, self, self.__class__)


    def InitCodeFor_XY_LinearDecayFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.linearDecayForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_LinearDecay, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With XY Linear Decay"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * x * y)"
        self.RequiresNonzeroX = 1
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_LinearDecay, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_LinearDecay, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_LinearDecay, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_LinearDecay, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_LinearDecay, self, self.__class__)


    def InitCodeFor_Y_LinearDecayAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.linearDecayAndOffsetForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_LinearDecayAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With Y Linear Decay And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * y) + " + self.additionalDesignatorList[1]
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_LinearDecayAndOffset, self, self.__class__)


    def InitCodeFor_XY_LinearDecayAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.linearDecayAndOffsetForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_LinearDecayAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With XY Linear Decay And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * x * y) + " + self.additionalDesignatorList[1]
        self.RequiresNonzeroX = 1
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_LinearDecayAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_LinearDecayAndOffset, self, self.__class__)


    def InitCodeFor_Y_ExponentialDecayFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.exponentialDecayForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_ExponentialDecay, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With Y Exponential Decay"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * e<SUP>y</SUP>)"
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_ExponentialDecay, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_ExponentialDecay, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_ExponentialDecay, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_ExponentialDecay, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_ExponentialDecay, self, self.__class__)


    def InitCodeFor_XY_ExponentialDecayFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.exponentialDecayForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_ExponentialDecay, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With XY Exponential Decay"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * e<SUP>x*y</SUP>)"
        self.RequiresNonzeroX = 1
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_ExponentialDecay, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_ExponentialDecay, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_ExponentialDecay, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_ExponentialDecay, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_ExponentialDecay, self, self.__class__)


    def InitCodeFor_Y_ExponentialDecayAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.exponentialDecayAndOffsetForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_ExponentialDecayAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With Y Exponential Decay And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * e<SUP>y</SUP>) + " + self.additionalDesignatorList[1]
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_ExponentialDecayAndOffset, self, self.__class__)


    def InitCodeFor_XY_ExponentialDecayAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.exponentialDecayAndOffsetForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_ExponentialDecayAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') / (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With XY Exponential Decay And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") / (" + self.additionalDesignatorList[0] + " * e<SUP>x*y</SUP>) + " + self.additionalDesignatorList[1]
        self.RequiresNonzeroX = 1
        self.RequiresNonzeroY = 1
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_ExponentialDecayAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_ExponentialDecayAndOffset, self, self.__class__)


    def InitCodeFor_Y_LinearGrowthFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.linearGrowthForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_LinearGrowth, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With Y Linear Growth"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * y)"
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_LinearGrowth, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_LinearGrowth, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_LinearGrowth, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_LinearGrowth, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_LinearGrowth, self, self.__class__)


    def InitCodeFor_XY_LinearGrowthFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.linearGrowthForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_LinearGrowth, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With XY Linear Growth"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * x * y)"
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_LinearGrowth, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_LinearGrowth, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_LinearGrowth, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_LinearGrowth, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_LinearGrowth, self, self.__class__)


    def InitCodeFor_Y_LinearGrowthAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.linearGrowthAndOffsetForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_LinearGrowthAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With Y Linear Growth And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * y) + " + self.additionalDesignatorList[1]
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_LinearGrowthAndOffset, self, self.__class__)


    def InitCodeFor_XY_LinearGrowthAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.linearGrowthAndOffsetForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_LinearGrowthAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With XY Linear Growth And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * x * y) + " + self.additionalDesignatorList[1]
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_LinearGrowthAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_LinearGrowthAndOffset, self, self.__class__)


    def InitCodeFor_Y_ExponentialGrowthFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.exponentialGrowthForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_ExponentialGrowth, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With Y Exponential Growth"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * e<SUP>y</SUP>)"
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_ExponentialGrowth, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_ExponentialGrowth, self, self.__class__)


    def InitCodeFor_XY_ExponentialGrowthFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(1)
        self.exponentialGrowthForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_ExponentialGrowth, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-1) + '] + i]);//')
        self.name += " With XY Exponential Growth"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * e<SUP>x*y</SUP>)"
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_ExponentialGrowth, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_ExponentialGrowth, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_ExponentialGrowth, self, self.__class__)


    def InitCodeFor_Y_ExponentialGrowthAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.exponentialGrowthAndOffsetForm_Y = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__Y_ExponentialGrowthAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With Y Exponential Growth And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * e<SUP>y</SUP>) + " + self.additionalDesignatorList[1]
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__Y_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__Y_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__Y_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__Y_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__Y_ExponentialGrowthAndOffset, self, self.__class__)


    def InitCodeFor_XY_ExponentialGrowthAndOffsetFormsOfEquations(self):
        self.CommonInitCodeForAdditionalForms(2)
        self.exponentialGrowthAndOffsetForm_XY = True
        self.CreateCacheGenerationList = new.instancemethod(ExtraCodeForEquationBaseClasses.CreateCacheGenerationList__XY_ExponentialGrowthAndOffset, self, self.__class__)
        if 0 == len(self.CacheGenerationList):
            self.CreateCacheGenerationList()
        self.function_cpp_code = self.function_cpp_code.replace("temp = ", "temp = (").replace(';//', ') * (coeff[' + str(self.previousNumberOfCoefficients) + '] * indep_data[(int)cachedWeaveOffsets[' + str(len(self.CacheGenerationList)-2) + '] + i]) + coeff[' + str(self.previousNumberOfCoefficients+1) + '];//')
        self.name += " With XY Exponential Growth And Offset"
        # some equations have a single line of HTML, others two.  This logic works for both
        index = self.HTML.rfind('= ') # the index of the rightmost '=' sign plus a space
        self.HTML = self.HTML[:index] + "= (" + self.HTML[index+1:] + ") * (" + self.additionalDesignatorList[0] + " * e<SUP>x*y</SUP>) + " + self.additionalDesignatorList[1]
        self.LinearSSQSolverFlag = 0
        # method overrides
        self.SpecificCodeCPP = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCPP__XY_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeJAVA = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeJAVA__XY_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeCS = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeCS__XY_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodePYTHON = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodePYTHON__XY_ExponentialGrowthAndOffset, self, self.__class__)
        self.SpecificCodeSCILAB = new.instancemethod(ExtraCodeForEquationBaseClasses.SpecificCodeSCILAB__XY_ExponentialGrowthAndOffset, self, self.__class__)




