#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id$
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import pythonequations, pythonequations.EquationBaseClasses
import numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them


class CombinedPowerAndExponential2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Combined Power And Exponential"
        self.HTML = "y = ax<sup>b</sup>e<sup>cx</sup>"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1]) * exp(coeff[2] * indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow(x_in, b) * exp(c * x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow(x_in, b) - Math.exp(c * x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow(x_in, b) - Math.Exp(c * x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a * math.pow(x_in, b) - math.xpc * x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a * pow(x_in, b) - exp(c * x_in)\n"
        return s


class CombinedPowerAndExponential_WO_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class CombinedPowerAndExponential_LinearDecay_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class CombinedPowerAndExponential_ExponentialDecay_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class CombinedPowerAndExponential_LinearGrowth_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class CombinedPowerAndExponential_ExponentialGrowth_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class CombinedPowerAndExponential_LinearDecayAndOffset_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class CombinedPowerAndExponential_ExponentialDecayAndOffset_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class CombinedPowerAndExponential_LinearGrowthAndOffset_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class CombinedPowerAndExponential_ExponentialGrowthAndOffset_2D(CombinedPowerAndExponential2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class Serpentine2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Serpentine"
        self.HTML = "y = ax / (1.0 + bx<sup>2</sup>)"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = coeff[0] * indep_data[(int)cachedWeaveOffsets[0] + i] / (1.0 + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * x_in / (1.0 + b * x_in * x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * x_in / (1.0 + b * x_in * x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * x_in / (1.0 + b * x_in * x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a * x_in / (1.0 + b * x_in * x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a * x_in / (1.0 + b * x_in * x_in)\n"
        return s


class Serpentine_WO_2D(Serpentine2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class Serpentine_LinearDecay_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class Serpentine_ExponentialDecay_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class Serpentine_LinearGrowth_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class Serpentine_ExponentialGrowth_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class Serpentine_LinearDecayAndOffset_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class Serpentine_ExponentialDecayAndOffset_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class Serpentine_LinearGrowthAndOffset_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class Serpentine_ExponentialGrowthAndOffset_2D(Serpentine2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class WitchOfAgnesiA2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Witch Of Maria Agnesi A"
        self.HTML = "y = 8a<sup>3</sup> / (x<sup>2</sup> + 4a<sup>2</sup>)"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = 8.0 * pow(coeff[0], 3.0) / (indep_data[(int)cachedWeaveOffsets[0] + i] + 4.0 * coeff[0] * coeff[0]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp = 8.0 * a * a * a / (x_in * x_in + 4.0 * a * a);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 8.0 * a * a * a / (x_in * x_in + 4.0 * a * a);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 8.0 * a * a * a / (x_in * x_in + 4.0 * a * a);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = 8.0 * a * a * a / (x_in * x_in + 4.0 * a * a)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = 8.0 * a * a * a / (x_in * x_in + 4.0 * a * a)\n"
        return s


class WitchOfAgnesiA_WO_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class WitchOfAgnesiA_LinearDecay_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class WitchOfAgnesiA_ExponentialDecay_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class WitchOfAgnesiA_LinearGrowth_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class WitchOfAgnesiA_ExponentialGrowth_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class WitchOfAgnesiA_LinearDecayAndOffset_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class WitchOfAgnesiA_ExponentialDecayAndOffset_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class WitchOfAgnesiA_LinearGrowthAndOffset_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class WitchOfAgnesiA_ExponentialGrowthAndOffset_2D(WitchOfAgnesiA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class WitchOfAgnesiB2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Witch Of Maria Agnesi B"
        self.HTML = "y = a<sup>3</sup> / (x<sup>2</sup> + a<sup>2</sup>)"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = pow(coeff[0], 3.0) / (indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[0] * coeff[0]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * a * a / (x_in * x_in + a * a);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * a * a / (x_in * x_in + a * a);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * a * a / (x_in * x_in + a * a);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a * a * a / (x_in * x_in + a * a)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a * a * a / (x_in * x_in + a * a)\n"
        return s


class WitchOfAgnesiB_WO_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class WitchOfAgnesiB_LinearDecay_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class WitchOfAgnesiB_ExponentialDecay_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class WitchOfAgnesiB_LinearGrowth_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class WitchOfAgnesiB_ExponentialGrowth_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class WitchOfAgnesiB_LinearDecayAndOffset_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class WitchOfAgnesiB_ExponentialDecayAndOffset_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class WitchOfAgnesiB_LinearGrowthAndOffset_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class WitchOfAgnesiB_ExponentialGrowthAndOffset_2D(WitchOfAgnesiB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class WitchOfAgnesiC2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Witch Of Maria Agnesi C"
        self.HTML = "y = a<sup>3</sup> / ((x * b + c)<sup>2</sup> + a<sup>2</sup>)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = pow(coeff[0], 3.0) / (pow(indep_data[(int)cachedWeaveOffsets[0] + i] * coeff[1] + coeff[2], 2.0) + coeff[0] * coeff[0]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * a * a / ((x_in * b + c) * (x_in * b + c) + a * a);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * a * a / ((x_in * b + c) * (x_in * b + c) + a * a);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * a * a / ((x_in * b + c) * (x_in * b + c) + a * a);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a * a * a / ((x_in * b + c) * (x_in * b + c) + a * a)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a * a * a / ((x_in * b + c) * (x_in * b + c) + a * a)\n"
        return s


class WitchOfAgnesiC_WO_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class WitchOfAgnesiC_LinearDecay_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class WitchOfAgnesiC_ExponentialDecay_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class WitchOfAgnesiC_LinearGrowth_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class WitchOfAgnesiC_ExponentialGrowth_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class WitchOfAgnesiC_LinearDecayAndOffset_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class WitchOfAgnesiC_ExponentialDecayAndOffset_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class WitchOfAgnesiC_LinearGrowthAndOffset_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class WitchOfAgnesiC_ExponentialGrowthAndOffset_2D(WitchOfAgnesiC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class RationalB2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Rational B"
        self.HTML = "y = a / (b + x + x<sup>2</sup>)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = coeff[0] / (coeff[1] + indep_data[(int)cachedWeaveOffsets[0] + i] + indep_data[(int)cachedWeaveOffsets[1] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp = a / (b + x_in + (x_in * x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a / (b + x_in + (x_in * x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a / (b + x_in + (x_in * x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a / (b + x_in + (x_in * x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a / (b + x_in + (x_in * x_in))\n"
        return s


class RationalB_WO_2D(RationalB2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class RationalB_LinearDecay_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class RationalB_ExponentialDecay_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class RationalB_LinearGrowth_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class RationalB_ExponentialGrowth_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class RationalB_LinearDecayAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class RationalB_ExponentialDecayAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class RationalB_LinearGrowthAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class RationalB_ExponentialGrowthAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class RationalA2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Rational A"
        self.HTML = "y = a / (b + x)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = coeff[0] / (coeff[1] + indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a / (b + x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a / (b + x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a / (b + x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a / (b + x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a / (b + x_in)\n"
        return s


class RationalA_WO_2D(RationalA2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class RationalA_LinearDecay_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class RationalA_ExponentialDecay_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class RationalA_LinearGrowth_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class RationalA_ExponentialGrowth_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class RationalA_LinearDecayAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class RationalA_ExponentialDecayAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class RationalA_LinearGrowthAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class RationalA_ExponentialGrowthAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class PursuitCurve2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Pursuit Curve"
        self.HTML = "y = ax<sup>2</sup> - log(x)"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = (coeff[0] * indep_data[(int)cachedWeaveOffsets[0] + i]) - indep_data[(int)cachedWeaveOffsets[1] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_LogX(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a - x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a - x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a - x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = 1.0 / (a - x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = 1.0 / (a - x_in)\n"
        return s


class PursuitCurve_WO_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class PursuitCurve_LinearDecay_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class PursuitCurve_ExponentialDecay_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class PursuitCurve_LinearGrowth_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class PursuitCurve_ExponentialGrowth_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class PursuitCurve_LinearDecayAndOffset_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class PursuitCurve_ExponentialDecayAndOffset_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class PursuitCurve_LinearGrowthAndOffset_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class PursuitCurve_ExponentialGrowthAndOffset_2D(PursuitCurve2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class PursuitCurve_Transform2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Pursuit Curve Transform"
        self.HTML = "y = a(bx + c)<sup>2</sup> - log(bx + c)"
        self.coefficientDesignatorTuple = ("a", 'b', 'c')

        self.function_cpp_code = 'temp = (coeff[0] * pow(coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2], 2.0)) - log(coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = (a * pow(b * x_in + c, 2.0)) - log(b * x_in + c);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = (a * Math.pow(b * x_in + c, 2.0)) - Math.log(b * x_in + c);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = (a * Math.Pow(b * x_in + c, 2.0)) - Math.Log(b * x_in + c);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = (a * math.pow(b * x_in + c, 2.0)) - math.log(b * x_in + c)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = (a * pow(b * x_in + c, 2.0)) - log(b * x_in + c)\n"
        return s


class PursuitCurve_Transform_WO_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class PursuitCurve_Transform_LinearDecay_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class PursuitCurve_Transform_ExponentialDecay_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class PursuitCurve_Transform_LinearGrowth_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class PursuitCurve_Transform_ExponentialGrowth_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class PursuitCurve_Transform_LinearDecayAndOffset_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class PursuitCurve_Transform_ExponentialDecayAndOffset_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class PursuitCurve_Transform_LinearGrowthAndOffset_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class PursuitCurve_Transform_ExponentialGrowthAndOffset_2D(PursuitCurve_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class AsymptoticPareto2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Asymptotic Pareto"
        self.HTML = "y = 1.0 - (1.0 / x<SUP>a</SUP>)"
        self.coefficientDesignatorTuple = ("a")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = 1.0 - (1.0 / (pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[0])));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 - (1.0 / (pow(x_in, a)));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 - (1.0 / (Math.pow(x_in, a)));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 - (1.0 / (Math.Pow(x_in, a)));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 - (1.0 / (math.pow(x_in, a)))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 - (1.0 / (x_in ^ a))\n"
        return s


class AsymptoticPareto_LinearDecay_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class AsymptoticPareto_ExponentialDecay_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class AsymptoticPareto_LinearGrowth_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class AsymptoticPareto_ExponentialGrowth_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class AsymptoticPareto_LinearDecayAndOffset_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class AsymptoticPareto_ExponentialDecayAndOffset_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class AsymptoticPareto_LinearGrowthAndOffset_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class AsymptoticPareto_ExponentialGrowthAndOffset_2D(AsymptoticPareto2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedSquare2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Square"
        self.HTML = "y = x<sup>2</sup> - ax"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[1] + i] - (coeff[0] * indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp = (x_in * x_in) - (a * x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = (x_in * x_in) - (a * x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = (x_in * x_in) - (a * x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = (x_in * x_in) - (a * x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = (x_in * x_in) - (a * x_in)\n"
        return s


class ModifiedSquare_WO_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedSquare_LinearDecay_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedSquare_ExponentialDecay_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedSquare_LinearGrowth_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedSquare_ExponentialGrowth_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedSquare_LinearDecayAndOffset_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedSquare_ExponentialDecayAndOffset_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedSquare_LinearGrowthAndOffset_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedSquare_ExponentialGrowthAndOffset_2D(ModifiedSquare2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedSquare_Transform2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Square Transform"
        self.HTML = "y = (bx + c)<sup>2</sup> - a(bx + c)"
        self.coefficientDesignatorTuple = ("a", 'b', 'c')

        self.function_cpp_code = 'temp = pow(coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2], 2.0) - (coeff[0] * (coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = pow(b * x_in + c, 2.0) - (a * (b * x_in + c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = Math.pow(b * x_in + c, 2.0) - (a * (b * x_in + c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = Math.Pow(b * x_in + c, 2.0) - (a * (b * x_in + c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = math.pow(b * x_in + c, 2.0) - (a * (b * x_in + c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = pow(b * x_in + c, 2.0) - (a * (b * x_in + c))\n"
        return s


class ModifiedSquare_Transform_WO_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedSquare_Transform_LinearDecay_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedSquare_Transform_ExponentialDecay_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedSquare_Transform_LinearGrowth_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedSquare_Transform_ExponentialGrowth_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedSquare_Transform_LinearDecayAndOffset_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedSquare_Transform_ExponentialDecayAndOffset_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedSquare_Transform_LinearGrowthAndOffset_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedSquare_Transform_ExponentialGrowthAndOffset_2D(ModifiedSquare_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ShiftedReciprocal2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Shifted Reciprocal"
        self.HTML = "y = 1.0 / (a - x)"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] - indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a - x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a - x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a - x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = 1.0 / (a - x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = 1.0 / (a - x_in)\n"
        return s


class ShiftedReciprocal_WO_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ShiftedReciprocal_LinearDecay_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ShiftedReciprocal_ExponentialDecay_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ShiftedReciprocal_LinearGrowth_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ShiftedReciprocal_ExponentialGrowth_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ShiftedReciprocal_LinearDecayAndOffset_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ShiftedReciprocal_ExponentialDecayAndOffset_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ShiftedReciprocal_LinearGrowthAndOffset_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ShiftedReciprocal_ExponentialGrowthAndOffset_2D(ShiftedReciprocal2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class Misc12D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Miscellaneous 1"
        self.HTML = "y = 1.0 + a(1.0 - e<SUP>bx</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = 1.0 + coeff[0] * (1.0 - exp(coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 + a * (1.0 - exp(b * x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 + a * (1.0 - Math.exp(b * x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 + a * (1.0 - Math.Exp(b * x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = 1.0 + a * (1.0 - math.exp(b * x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = 1.0 + a * (1.0 - exp(b * x_in))\n"
        return s


class Misc1_WO_2D(Misc12D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class Misc1_LinearDecay_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class Misc1_ExponentialDecay_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class Misc1_LinearGrowth_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class Misc1_ExponentialGrowth_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class Misc1_LinearDecayAndOffset_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class Misc1_ExponentialDecayAndOffset_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class Misc1_LinearGrowthAndOffset_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class Misc1_ExponentialGrowthAndOffset_2D(Misc12D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class DoubleLangmuirProbeCharacteristic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Double Langmuir Probe Characteristic"
        self.HTML = "y = a(tanh(bx+c))"
        self.coefficientDesignatorTuple = ('a', 'b', 'c')

        self.function_cpp_code = 'temp = coeff[0] * (tanh((coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]) + coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * (tanh((b * x_in) + c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * (Math.tanh((b * x_in) + c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * (Math.Tanh((b * x_in) + c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\t\ttemp = a * (math.tanh((b * x_in) + c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\t\ttemp = a * (tanh((b * x_in) + c))\n"
        return s


class DoubleLangmuirProbeCharacteristic_WO_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_LinearDecay_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class BDoubleLangmuirProbeCharacteristic_ExponentialDecay_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_LinearGrowth_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_ExponentialGrowth_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_LinearDecayAndOffset_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_ExponentialDecayAndOffset_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_LinearGrowthAndOffset_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class DoubleLangmuirProbeCharacteristic_ExponentialGrowthAndOffset_2D(DoubleLangmuirProbeCharacteristic2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()
