#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id$
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import pythonequations, pythonequations.EquationBaseClasses
import numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them

class BoxLucas2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Box Lucas"
        self.HTML = "y = (a / (a-b)) * (e<SUP>(-bx)</sup> - e<SUP>(-ax)</sup>)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = (coeff[0] / (coeff[0] - coeff[1])) * (exp(-1.0 * coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]) - exp(-1.0 * coeff[0] * indep_data[(int)cachedWeaveOffsets[0] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = (a / (a-b)) * (exp(-1.0 * b *x_in) - exp(-1.0 * a * x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = (a / (a-b)) * (Math.exp(-1.0 * b *x_in)) - Math.exp(-1.0 * a *x_in)));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = (a / (a-b)) * (Math.Exp(-1.0 * b *x_in)) - Math.Exp(-1.0 * a *x_in)));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = (a / (a-b)) * (math.exp(-1.0 * b *x_in)) - math.exp(-1.0 * a *x_in)))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = (a / (a-b)) * (exp(-1.0 * b *x_in)) - exp(-1.0 * a *x_in)))\n"
        return s


class BoxLucas_WO_2D(BoxLucas2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class BoxLucas_LinearDecay_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class BoxLucas_ExponentialDecay_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class BoxLucas_LinearGrowth_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class BoxLucas_ExponentialGrowth_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class BoxLucas_LinearDecayAndOffset_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class BoxLucas_ExponentialDecayAndOffset_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class BoxLucas_LinearGrowthAndOffset_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class BoxLucas_ExponentialGrowthAndOffset_2D(BoxLucas2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class Gaussian2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Gaussian Peak"
        self.HTML = "y = a * e<SUP>(-0.5 * (x-b)^2 / c^2)</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = coeff[0] * exp(-0.5 * (indep_data[(int)cachedWeaveOffsets[1] + i] - coeff[1]) * (indep_data[(int)cachedWeaveOffsets[1] + i] - coeff[1]) / (coeff[2] * coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * exp(-0.5 * (x_in-b) * (x_in-b) / (c*c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.exp(-0.5 * (x_in-b) * (x_in-b) / (c*c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Exp(-0.5 * (x_in-b) * (x_in-b) / (c*c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.exp(-0.5 * (x_in-b) * (x_in-b) / (c*c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * exp(-0.5 * (x_in-b) * (x_in-b) / (c*c))\n"
        return s


class Gaussian_WO_2D(Gaussian2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class Gaussian_LinearDecay_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class Gaussian_ExponentialDecay_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class Gaussian_LinearGrowth_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class Gaussian_ExponentialGrowth_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class Gaussian_LinearDecayAndOffset_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class Gaussian_ExponentialDecayAndOffset_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class Gaussian_LinearGrowthAndOffset_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class Gaussian_ExponentialGrowthAndOffset_2D(Gaussian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class Lorentzian2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Lorentzian Peak"
        self.HTML = "y = a / (1.0 + ((x-b)/c)<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = coeff[0] / (1.0 + (indep_data[(int)cachedWeaveOffsets[1] + i] - coeff[1]) * (indep_data[(int)cachedWeaveOffsets[1] + i] - coeff[1]) / (coeff[2] * coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a / (1.0 + pow((x_in-b)/c, 2.0));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\ttemp = a / (1.0 + Math.pow((x_in-b)/c, 2.0));\n"
        return s

    def SpecificCodeCS(self):
        s = "\ttemp = a / (1.0 + Math.Pow((x_in-b)/c, 2.0));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a / (1.0 + math.pow((x_in-b)/c, 2.0))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a / (1.0 + ((x_in-b)/c ^ 2.0))\n"
        return s


class Lorentzian_WO_2D(Lorentzian2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class Lorentzian_LinearDecay_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class Lorentzian_ExponentialDecay_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class Lorentzian_LinearGrowth_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class Lorentzian_ExponentialGrowth_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class Lorentzian_LinearDecayAndOffset_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class Lorentzian_ExponentialDecayAndOffset_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class Lorentzian_LinearGrowthAndOffset_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class Lorentzian_ExponentialGrowthAndOffset_2D(Lorentzian2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class Pulse2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Pulse Peak"
        self.HTML = "y = 4.0 * a * e<SUP>(-(x-b)/c)</SUP> * (1.0 - e<SUP>(-(x-b)/c)</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = 4.0 * coeff[0] * exp(-1.0 * (indep_data[(int)cachedWeaveOffsets[1] + i] - coeff[1]) / coeff[2]) * (1.0 - exp(-1.0 * (indep_data[(int)cachedWeaveOffsets[1] + i] - coeff[1]) / coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 4.0 * a * exp(-1.0 * (x_in-b) / c) * (1.0 - exp(-1.0 * (x_in-b) / c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\ttemp = 4.0 * a * Math.exp(-1.0 * (x_in-b) / c) * (1.0 - Math.exp(-1.0 * (x_in-b) / c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\ttemp = 4.0 * a * Math.Exp(-1.0 * (x_in-b) / c) * (1.0 - Math.Exp(-1.0 * (x_in-b) / c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 4.0 * a * math.exp(-1.0 * (x_in-b) / c) * (1.0 - math.exp(-1.0 * (x_in-b) / c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 4.0 * a * exp(-1.0 * (x_in-b) / c) * (1.0 - exp(-1.0 * (x_in-b) / c))\n"
        return s


class Pulse_WO_2D(Pulse2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class Pulse_LinearDecay_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class Pulse_ExponentialDecay_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class Pulse_LinearGrowth_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class Pulse_ExponentialGrowth_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class Pulse_LinearDecayAndOffset_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class Pulse_ExponentialDecayAndOffset_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class Pulse_LinearGrowthAndOffset_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class Pulse_ExponentialGrowthAndOffset_2D(Pulse2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()
