#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id$
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import pythonequations, pythonequations.EquationBaseClasses
import numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them

class MarcPlanteQuadratic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Marc Plante's Custom Quadratic"
        self.HTML = "y = (-b + (b<sup>2</sup> - 4 a (c - x))<sup>0.5</sup>) / 2 / a "
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = ((-1.0 * coeff[1]) + pow((coeff[1] * coeff[1]) - 4.0 * coeff[0] * (coeff[2] - indep_data[(int)cachedWeaveOffsets[0] + i]), 0.5)) / 2.0 / coeff[0];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += ((-1.0 * " + self.coefficientDesignatorTuple[1] + ") + pow((" + self.coefficientDesignatorTuple[1] + " * " + self.coefficientDesignatorTuple[1] + ") - 4.0 * " + self.coefficientDesignatorTuple[0]  + " * (" + self.coefficientDesignatorTuple[2] + " - x_in), 0.5)) / 2.0 / " + self.coefficientDesignatorTuple[1] + ";\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += ((-1.0 * " + self.coefficientDesignatorTuple[1] + ") + Math.pow((" + self.coefficientDesignatorTuple[1] + " * " + self.coefficientDesignatorTuple[1] + ") - 4.0 * " + self.coefficientDesignatorTuple[0]  + " * (" + self.coefficientDesignatorTuple[2] + " - x_in), 0.5)) / 2.0 / " + self.coefficientDesignatorTuple[1] + ";\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += ((-1.0 * " + self.coefficientDesignatorTuple[1] + ") + Math.Pow((" + self.coefficientDesignatorTuple[1] + " * " + self.coefficientDesignatorTuple[1] + ") - 4.0 * " + self.coefficientDesignatorTuple[0]  + " * (" + self.coefficientDesignatorTuple[2] + " - x_in), 0.5)) / 2.0 / " + self.coefficientDesignatorTuple[1] + ";\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += ((-1.0 * " + self.coefficientDesignatorTuple[1] + ") + math.pow((" + self.coefficientDesignatorTuple[1] + " * " + self.coefficientDesignatorTuple[1] + ") - 4.0 * " + self.coefficientDesignatorTuple[0]  + " * (" + self.coefficientDesignatorTuple[2] + " - x_in), 0.5)) / 2.0 / " + self.coefficientDesignatorTuple[1] + "\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp += ((-1.0 * " + self.coefficientDesignatorTuple[1] + ") + ((" + self.coefficientDesignatorTuple[1] + " * " + self.coefficientDesignatorTuple[1] + ") - 4.0 * " + self.coefficientDesignatorTuple[0]  + " * (" + self.coefficientDesignatorTuple[2] + " - x_in)) ^ 0.5) / 2.0 / " + self.coefficientDesignatorTuple[1] + "\n"
        return s


class Linear2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Linear"
        self.HTML = "y = a + bx"
        self.coefficientDesignatorTuple = ("a", "b")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + (coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        return s


class ReciprocalLinear2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Linear"
        self.HTML = "y = 1.0 / (a + bx)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + (coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += 1.0 / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += 1.0 / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += 1.0 / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += 1.0 / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + (1.0 / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in))\n"
        return s


class InverseLinear2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Inverse Linear"
        self.HTML = "y = x / (a + bx)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[0] + i] / (coeff[0] + (coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1] + "\n"


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += x_in / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += x_in / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += x_in / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += x_in / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + (x_in / (" + self.coefficientDesignatorTuple[0] + " + " + self.coefficientDesignatorTuple[1] + " * x_in))\n"
        return s


class Quadratic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Quadratic"
        self.HTML = "y = a + bx + cx<SUP>2</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1),  []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * pow(x_in, 2.0);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.pow(x_in, 2.0);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.Pow(x_in, 2.0);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * math.pow(x_in, 2.0)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * x_in ^ 2.0\n"
        return s


class ReciprocalQuadratic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Quadratic"
        self.HTML = "y = 1.0 / (a + bx + cx<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1),  []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * pow(x_in, 2.0);\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * x_in ^ 2.0\n"
        s += "\ttemp = 1.0 / temp\n"
        return s


class InverseQuadratic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Inverse Quadratic"
        self.HTML = "y = x / (a + bx + cx<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[0] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[1] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1),  []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * pow(x_in, 2.0);\n"
        s += "\ttemp = x_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp = x_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp = x_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp = x_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * x_in ^ 2.0\n"
        s += "\ttemp = x_in / temp\n"
        return s


class Cubic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Cubic"
        self.HTML = "y = a + bx + cx<SUP>2</SUP> + dx<SUP>3</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(2)]), [float(2)]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(3)]), [float(3)]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 3.0);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 3.0);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 3.0);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 3.0)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp +" + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[2] + " * x_in ^ 2.0\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[3] + " * x_in ^ 3.0\n"
        return s


class ReciprocalCubic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Cubic"
        self.HTML = "y = 1.0 / (a + bx + cx<SUP>2</SUP> + dx<SUP>3</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(2)]), [float(2)]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(3)]), [float(3)]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 3.0);\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp +" + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[2] + " * x_in ^ 2.0\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[3] + " * x_in ^ 3.0\n"
        s += "\ttemp = 1.0 / temp\n"
        return s


class InverseCubic2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Inverse Cubic"
        self.HTML = "y = x / (a + bx + cx<SUP>2</SUP> + dx<SUP>3</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[1] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(2)]), [float(2)]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(3)]), [float(3)]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 3.0);\n"
        s += "\ttemp = x_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp = x_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp = x_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp = x_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp +" + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[2] + " * x_in ^ 2.0\n"
        s += "\ttemp = temp +" + self.coefficientDesignatorTuple[3] + " * x_in ^ 3.0\n"
        s += "\ttemp = x_in / temp\n"
        return s


class Polynomial2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "User-Selectable Polynomial"
        self.HTML = "y = a + bx + cx<SUP>2</SUP> + dx<SUP>3</SUP> + ..."
        self.coefficientDesignatorTuple = ()
        self.polynomialFlag = 1
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = 0.0;\nfor (int k=0; k<numberOfCoefficients; k++)\n\ttemp += coeff[k] * indep_data[(int)cachedWeaveOffsets[k] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(i)]), [float(i)]])

    def SpecificCodeCPP(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        return s

    def SpecificCodeJAVA(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\t\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\t\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        return s

    def SpecificCodeCS(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\t\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\t\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        return s

    def SpecificCodePYTHON(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + "\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + "\n"
        return s

    def SpecificCodeSCILAB(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + "\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + "\n"
        return s

    def RegenerateForm(self):
        self.coefficientDesignatorTuple = ()
        self.HTML = "y = "
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[i]),
            if i == 0:
                self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[i]
            else:
                self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[i] + 'x<SUP>' + `i` + '</SUP>'
            if i != self.xPolynomialOrder:
                self.HTML += ' + '

    def Initialize(self):
        self.coefficientDesignatorTuple = ()
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[i]),
        pythonequations.EquationBaseClasses.Equation2D.Initialize(self)


class ReciprocalPolynomial2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "User-Selectable Reciprocal Polynomial"
        self.HTML = "y = 1.0 / (a + bx + cx<SUP>2</SUP> + dx<SUP>3</SUP> + ...)"
        self.coefficientDesignatorTuple = ()
        self.polynomialFlag = 1

        self.function_cpp_code  = 'temp = 0.0;\nfor (int k=0; k<numberOfCoefficients; k++)\n'
        self.function_cpp_code += '\ttemp += coeff[k] * indep_data[(int)cachedWeaveOffsets[k] + i];\n'
        self.function_cpp_code += 'temp = 1.0 / temp;// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(i)]), [float(i)]])

    def SpecificCodeCPP(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\t\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\t\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\t\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\t\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + "\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + "\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + "\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + "\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def RegenerateForm(self):
        self.coefficientDesignatorTuple = ()
        self.HTML = "y = 1.0 / ("
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[i]),
            if i == 0:
                self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[i]
            else:
                self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[i] + 'x<SUP>' + `i` + '</SUP>'
            if i != self.xPolynomialOrder:
                self.HTML += ' + '
        self.HTML += ")"

    def Initialize(self):
        self.coefficientDesignatorTuple = ()
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[i]),
        pythonequations.EquationBaseClasses.Equation2D.Initialize(self)


class InversePolynomial2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "User-Selectable Inverse Polynomial"
        self.HTML = "y = x / (a + bx + cx<SUP>2</SUP> + dx<SUP>3</SUP> + ...)"
        self.coefficientDesignatorTuple = ()
        self.polynomialFlag = 1

        self.function_cpp_code  = 'temp = 0.0;\nfor (int k=0; k<numberOfCoefficients; k++)\n'
        self.function_cpp_code += '\ttemp += coeff[k] * indep_data[(int)cachedWeaveOffsets[k] + i];\n'
        self.function_cpp_code += 'temp = indep_data[(int)cachedWeaveOffsets[numberOfCoefficients] + i] / temp;// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[float(i)]), [float(i)]])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        s += "\ttemp = x_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\t\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\t\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        s += "\t\ttemp = x_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\t\ttemp = " + self.coefficientDesignatorTuple[i] + ";\n"
            else:
                s += "\t\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + ";\n"
        s += "\t\ttemp = x_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + "\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + "\n"
        s += "\ttemp = x_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = ""
        length = len(self.coefficientArray)
        for i in range(length-1, -1, -1):
            if i == length-1:
                s += "\ttemp = " + self.coefficientDesignatorTuple[i] + "\n"
            else:
                s += "\ttemp = temp * x_in + " + self.coefficientDesignatorTuple[i] + "\n"
        s += "\ttemp = x_in / temp\n"
        return s

    def RegenerateForm(self):
        self.coefficientDesignatorTuple = ()
        self.HTML = "y = x / ("
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[i]),
            if i == 0:
                self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[i]
            else:
                self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[i] + 'x<SUP>' + `i` + '</SUP>'
            if i != self.xPolynomialOrder:
                self.HTML += ' + '
        self.HTML += ")"

    def Initialize(self):
        self.coefficientDesignatorTuple = ()
        for i in range(self.xPolynomialOrder+1): # 0 - xOrder
            self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[i]),
        pythonequations.EquationBaseClasses.Equation2D.Initialize(self)


