#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id$
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import pythonequations, pythonequations.EquationBaseClasses
import numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them

class PowerRoot2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Root"
        self.HTML = "y = a<SUP>(1.0/x)</SUP>"
        self.coefficientDesignatorTuple = ("a")
        self.RequiresNonzeroX = 1

        self.function_cpp_code = 'temp = pow(coeff[0], indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_InverseX(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = pow(a, (1.0/x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = Math.pow(a, (1.0/x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = Math.Pow(a, (1.0/x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = math.pow(a, (1.0/x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = (a ^ (1.0/x_in))\n"
        return s


class PowerRoot_WO_2D(PowerRoot2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class PowerRoot_LinearDecay_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class PowerRoot_ExponentialDecay_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class PowerRoot_LinearGrowth_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class PowerRoot_ExponentialGrowth_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class PowerRoot_LinearDecayAndOffset_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class PowerRoot_ExponentialDecayAndOffset_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class PowerRoot_LinearGrowthAndOffset_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class PowerRoot_ExponentialGrowthAndOffset_2D(PowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedGeometric2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Geometric"
        self.HTML = "y = a * x<SUP>(b/x)</SUP>"
        self.coefficientDesignatorTuple = ("a", "b")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_InverseX(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow(x_in, (b/x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow(x_in, (b/x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow(x_in, (b/x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.pow(x_in, (b/x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * (x_in ^ (b/x_in))\n"
        return s


class ModifiedGeometric_WO_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedGeometric_LinearDecay_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedGeometric_ExponentialDecay_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedGeometric_LinearGrowth_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedGeometric_ExponentialGrowth_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedGeometric_LinearDecayAndOffset_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedGeometric_ExponentialDecayAndOffset_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedGeometric_LinearGrowthAndOffset_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedGeometric_ExponentialGrowthAndOffset_2D(ModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class StandardGeometric2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Standard Geometric"
        self.HTML = "y = a * x<SUP>bx</SUP>"
        self.coefficientDesignatorTuple = ("a", "b")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow(x_in, (b*x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow(x_in, (b*x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow(x_in, (b*x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.pow(x_in, (b*x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * (x_in ^ (b*x_in))\n"
        return s


class StandardGeometric_WO_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class StandardGeometric_LinearDecay_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class StandardGeometric_ExponentialDecay_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class StandardGeometric_LinearGrowth_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class StandardGeometric_ExponentialGrowth_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class StandardGeometric_LinearDecayAndOffset_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class StandardGeometric_ExponentialDecayAndOffset_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class StandardGeometric_LinearGrowthAndOffset_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class StandardGeometric_ExponentialGrowthAndOffset_2D(StandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class XShiftedPower2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "X Shifted Power"
        self.HTML = "y = a * (x-b)<SUP>c</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i] - coeff[1], coeff[2]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow((x_in-b), c);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow((x_in-b), c);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow((x_in-b), c);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.pow((x_in-b), c)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * ((x_in-b) ^ c)\n"
        return s


class XShiftedPower_WO_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class XShiftedPower_LinearDecay_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class XShiftedPower_ExponentialDecay_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class XShiftedPower_LinearGrowth_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class XShiftedPower_ExponentialGrowth_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class XShiftedPower_LinearDecayAndOffset_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class XShiftedPower_ExponentialDecayAndOffset_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class XShiftedPower_LinearGrowthAndOffset_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class XShiftedPower_ExponentialGrowthAndOffset_2D(XShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class SimplePower2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Simple Power"
        self.HTML = "y = x<SUP>a</SUP>"
        self.coefficientDesignatorTuple = ("a")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[0]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = pow(x_in, a);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = Math.pow(x_in, a);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = Math.Pow(x_in, a);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = math.pow(x_in, a)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = x_in ^ a\n"
        return s


class SimplePower_WO_2D(SimplePower2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class SimplePower_LinearDecay_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class SimplePower_ExponentialDecay_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class SimplePower_LinearGrowth_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class SimplePower_ExponentialGrowth_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class SimplePower_LinearDecayAndOffset_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class SimplePower_ExponentialDecayAndOffset_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class SimplePower_LinearGrowthAndOffset_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class SimplePower_ExponentialGrowthAndOffset_2D(SimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class StandardPower2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Standard Power"
        self.HTML = "y = a * x<SUP>b</SUP>"
        self.coefficientDesignatorTuple = ("a", "b")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow(x_in, b);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow(x_in, b);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow(x_in, b);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.pow(x_in, b)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * (x_in ^ b)\n"
        return s


class StandardPower_WO_2D(StandardPower2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class StandardPower_LinearDecay_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class StandardPower_ExponentialDecay_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class StandardPower_LinearGrowth_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class StandardPower_ExponentialGrowth_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class StandardPower_LinearDecayAndOffset_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class StandardPower_ExponentialDecayAndOffset_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class StandardPower_LinearGrowthAndOffset_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class StandardPower_ExponentialGrowthAndOffset_2D(StandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedPowerA2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Power A"
        self.HTML = "y = a * b<SUP>x</SUP>"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = coeff[0] * pow(coeff[1], indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow(b, x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow(b, x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow(b, x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.pow(b, x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * (b ^ x_in)\n"
        return s


class ModifiedPowerA_WO_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedPowerA_LinearDecay_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedPowerA_ExponentialDecay_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedPowerA_LinearGrowth_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedPowerA_ExponentialGrowth_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedPowerA_LinearDecayAndOffset_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedPowerA_ExponentialDecayAndOffset_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedPowerA_LinearGrowthAndOffset_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedPowerA_ExponentialGrowthAndOffset_2D(ModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedPowerA_Transform2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Power A Transform"
        self.HTML = "y = a * b<SUP>cx + d</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", 'c', 'd')

        self.function_cpp_code = 'temp = coeff[0] * pow(coeff[1], coeff[2] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[3]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a * pow(b, c * x_in + d);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a * Math.pow(b, c * x_in + d);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a * Math.Pow(b, c * x_in + d);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a * math.pow(b, c * x_in + d)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a * (b ^ (c * x_in + d))\n"
        return s


class ModifiedPowerA_Transform_WO_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedPowerA_Transform_LinearDecay_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedPowerA_Transform_ExponentialDecay_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedPowerA_Transform_LinearGrowth_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedPowerA_Transform_ExponentialGrowth_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedPowerA_Transform_LinearDecayAndOffset_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedPowerA_Transform_ExponentialDecayAndOffset_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedPowerA_Transform_LinearGrowthAndOffset_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedPowerA_Transform_ExponentialGrowthAndOffset_2D(ModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedPowerB2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Power B"
        self.HTML = "y = a<SUP>ln(x)</SUP>"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = pow(coeff[0], indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_LogX(NameOrValueFlag=1), []])
    
    def SpecificCodeCPP(self):
        s = "\ttemp = pow(a, log(x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = Math.pow(a, Math.log(x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = Math.Pow(a, Math.Log(x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = math.pow(a, math.log(x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a ^ log(x_in)\n"
        return s


class ModifiedPowerB_WO_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedPowerB_LinearDecay_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedPowerB_ExponentialDecay_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedPowerB_LinearGrowth_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedPowerB_ExponentialGrowth_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedPowerB_LinearDecayAndOffset_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedPowerB_ExponentialDecayAndOffset_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedPowerB_LinearGrowthAndOffset_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedPowerB_ExponentialGrowthAndOffset_2D(ModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedPowerB_Transform2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Power B Transform"
        self.HTML = "y = a<SUP>ln(bx + c)</SUP>"
        self.coefficientDesignatorTuple = ("a", 'b', 'c')

        self.function_cpp_code = 'temp = pow(coeff[0], log(coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
    
    def SpecificCodeCPP(self):
        s = "\ttemp = pow(a, log(b * x_in + c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = Math.pow(a, Math.log(b * x_in + c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = Math.Pow(a, Math.Log(b * x_in + c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = math.pow(a, math.log(b * x_in + c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a ^ log(b * x_in + c)\n"
        return s


class ModifiedPowerB_Transform_WO_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedPowerB_Transform_LinearDecay_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedPowerB_Transform_ExponentialDecay_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedPowerB_Transform_LinearGrowth_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedPowerB_Transform_ExponentialGrowth_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedPowerB_Transform_LinearDecayAndOffset_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedPowerB_Transform_ExponentialDecayAndOffset_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedPowerB_Transform_LinearGrowthAndOffset_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedPowerB_Transform_ExponentialGrowthAndOffset_2D(ModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ModifiedPowerC2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Modified Power C"
        self.HTML = "y = (a + x)<SUP>b</SUP>"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = pow(coeff[0] + indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
    
    def SpecificCodeCPP(self):
        s = "\ttemp = pow(a + x_in, b);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = Math.pow(a + x_in, b);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = Math.Pow(a + x_in, b);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = math.pow(a + x_in, b)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = (a + x_in) ^ b\n"
        return s


class ModifiedPowerC_WO_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ModifiedPowerC_LinearDecay_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ModifiedPowerC_ExponentialDecay_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ModifiedPowerC_LinearGrowth_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ModifiedPowerC_ExponentialGrowth_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ModifiedPowerC_LinearDecayAndOffset_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ModifiedPowerC_ExponentialDecayAndOffset_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ModifiedPowerC_LinearGrowthAndOffset_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ModifiedPowerC_ExponentialGrowthAndOffset_2D(ModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class PowerLawExponentialCutoff2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Power Law With Exponential Cutoff"
        self.HTML = "p(k) = C * k<SUP>(-T)</SUP> * e<SUP>(-k/K)</SUP>"
        self.coefficientDesignatorTuple = ("C", "T", "K")

        self.function_cpp_code = 'temp = coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[2] + i], -1.0 * coeff[1]) * exp(-1.0 * indep_data[(int)cachedWeaveOffsets[2] + i] / coeff[2]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = C * pow(x_in, -1.0 * T) * exp(-1.0 * x_in / K);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = C * Math.pow(x_in, -1.0 * T) * Math.exp(-1.0 * x_in / K);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = C * Math.Pow(x_in, -1.0 * T) * Math.Exp(-1.0 * x_in / K);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = C * pow(x_in, -1.0 * T) * exp(-1.0 * x_in / K)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = C * x_in ^ (-1.0 * T) * exp(-1.0 * x_in / K)\n"
        return s


class PowerLawExponentialCutoff_WO_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class PowerLawExponentialCutoff_LinearDecay_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class PowerLawExponentialCutoff_ExponentialDecay_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class PowerLawExponentialCutoff_LinearGrowth_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class PowerLawExponentialCutoff_ExponentialGrowth_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class PowerLawExponentialCutoff_LinearDecayAndOffset_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class PowerLawExponentialCutoff_ExponentialDecayAndOffset_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class PowerLawExponentialCutoff_LinearGrowthAndOffset_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class PowerLawExponentialCutoff_ExponentialGrowthAndOffset_2D(PowerLawExponentialCutoff2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()



class ReciprocalPowerRoot2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Root"
        self.HTML = "y = 1.0 / a<SUP>(1.0/x)</SUP>)"
        self.coefficientDesignatorTuple = ("a")
        self.RequiresNonzeroX = 1

        self.function_cpp_code = 'temp = 1.0 / (pow(coeff[0], indep_data[(int)cachedWeaveOffsets[0] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_InverseX(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (pow(a, (1.0/x_in)));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (Math.pow(a, (1.0/x_in)));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (Math.Pow(a, (1.0/x_in)));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (math.pow(a, (1.0/x_in)))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / ((a ^ (1.0/x_in)))\n"
        return s


class ReciprocalPowerRoot_WO_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalPowerRoot_LinearDecay_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalPowerRoot_ExponentialDecay_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalPowerRoot_LinearGrowth_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalPowerRoot_ExponentialGrowth_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalPowerRoot_LinearDecayAndOffset_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalPowerRoot_ExponentialDecayAndOffset_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalPowerRoot_LinearGrowthAndOffset_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalPowerRoot_ExponentialGrowthAndOffset_2D(ReciprocalPowerRoot2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalModifiedGeometric2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Modified Geometric"
        self.HTML = "y = 1.0 / (a * x<SUP>(b/x)</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_InverseX(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a * pow(x_in, (b/x_in)));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a * Math.pow(x_in, (b/x_in)));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a * Math.Pow(x_in, (b/x_in)));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (a * math.pow(x_in, (b/x_in)))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (a * (x_in ^ (b/x_in)))\n"
        return s


class ReciprocalModifiedGeometric_WO_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalModifiedGeometric_LinearDecay_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalModifiedGeometric_ExponentialDecay_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalModifiedGeometric_LinearGrowth_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalModifiedGeometric_ExponentialGrowth_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalModifiedGeometric_LinearDecayAndOffset_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedGeometric_ExponentialDecayAndOffset_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedGeometric_LinearGrowthAndOffset_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalModifiedGeometric_ExponentialGrowthAndOffset_2D(ReciprocalModifiedGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalStandardGeometric2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Standard Geometric"
        self.HTML = "y = 1.0 / (a * x<SUP>bx</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a * pow(x_in, (b*x_in)));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a * Math.pow(x_in, (b*x_in)));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a * Math.Pow(x_in, (b*x_in)));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (a * math.pow(x_in, (b*x_in)))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (a * (x_in ^ (b*x_in)))\n"
        return s


class ReciprocalStandardGeometric_WO_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalStandardGeometric_LinearDecay_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalStandardGeometric_ExponentialDecay_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalStandardGeometric_LinearGrowth_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalStandardGeometric_ExponentialGrowth_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalStandardGeometric_LinearDecayAndOffset_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalStandardGeometric_ExponentialDecayAndOffset_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalStandardGeometric_LinearGrowthAndOffset_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalStandardGeometric_ExponentialGrowthAndOffset_2D(ReciprocalStandardGeometric2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalXShiftedPower2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal X Shifted Power"
        self.HTML = "y = 1.0 / (a * (x-b)<SUP>c</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i] - coeff[1], coeff[2]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a * pow((x_in-b), c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a * Math.pow((x_in-b), c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a * Math.Pow((x_in-b), c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (a * math.pow((x_in-b), c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (a * ((x_in-b) ^ c))\n"
        return s


class ReciprocalXShiftedPower_WO_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalXShiftedPower_LinearDecay_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalXShiftedPower_ExponentialDecay_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalXShiftedPower_LinearGrowth_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalXShiftedPower_ExponentialGrowth_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalXShiftedPower_LinearDecayAndOffset_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalXShiftedPower_ExponentialDecayAndOffset_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalXShiftedPower_LinearGrowthAndOffset_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalXShiftedPower_ExponentialGrowthAndOffset_2D(ReciprocalXShiftedPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalSimplePower2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Simple Power"
        self.HTML = "y = 1.0 / x<SUP>a</SUP>"
        self.coefficientDesignatorTuple = ("a")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = 1.0 / (pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[0]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (pow(x_in, a));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (Math.pow(x_in, a));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (Math.Pow(x_in, a));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (math.pow(x_in, a))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (x_in ^ a)\n"
        return s


class ReciprocalSimplePower_WO_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalSimplePower_LinearDecay_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalSimplePower_ExponentialDecay_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalSimplePower_LinearGrowth_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalSimplePower_ExponentialGrowth_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalSimplePower_LinearDecayAndOffset_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalSimplePower_ExponentialDecayAndOffset_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalSimplePower_LinearGrowthAndOffset_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalSimplePower_ExponentialGrowthAndOffset_2D(ReciprocalSimplePower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalStandardPower2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Standard Power"
        self.HTML = "y = 1.0 / (a * x<SUP>b</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b")
        self.RequiresPositiveX = 1

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] * pow(indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a * pow(x_in, b));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a * Math.pow(x_in, b));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a * Math.Pow(x_in, b));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (a * math.pow(x_in, b))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (a * (x_in ^ b))\n"
        return s


class ReciprocalStandardPower_WO_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalStandardPower_LinearDecay_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalStandardPower_ExponentialDecay_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalStandardPower_LinearGrowth_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalStandardPower_ExponentialGrowth_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalStandardPower_LinearDecayAndOffset_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalStandardPower_ExponentialDecayAndOffset_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalStandardPower_LinearGrowthAndOffset_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalStandardPower_ExponentialGrowthAndOffset_2D(ReciprocalStandardPower2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalModifiedPowerA2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Modified Power A"
        self.HTML = "y = 1.0 / (a * b<SUP>x</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] * pow(coeff[1], indep_data[(int)cachedWeaveOffsets[0] + i]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a * pow(b, x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a * Math.pow(b, x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a * Math.Pow(b, x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (a * math.pow(b, x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (a * (b ^ x_in))\n"
        return s


class ReciprocalModifiedPowerA_WO_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_LinearDecay_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalModifiedPowerA_ExponentialDecay_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalModifiedPowerA_LinearGrowth_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalModifiedPowerA_ExponentialGrowth_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalModifiedPowerA_LinearDecayAndOffset_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_ExponentialDecayAndOffset_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_LinearGrowthAndOffset_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_ExponentialGrowthAndOffset_2D(ReciprocalModifiedPowerA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalModifiedPowerA_Transform2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Modified Power A Transform"
        self.HTML = "y = 1.0 / (a * b<SUP>cx + d</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", 'c', 'd')

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] * pow(coeff[1], coeff[2] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[3]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (a * pow(b, c * x_in + d));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (a * Math.pow(b, c * x_in + d));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (a * Math.Pow(b, c * x_in + d));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (a * math.pow(b, c * x_in + d))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / (a * (b ^ (c * x_in + d)))\n"
        return s


class ReciprocalModifiedPowerA_Transform_WO_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_LinearDecay_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_ExponentialDecay_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_LinearGrowth_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_ExponentialGrowth_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_LinearDecayAndOffset_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_ExponentialDecayAndOffset_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_LinearGrowthAndOffset_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerA_Transform_ExponentialGrowthAndOffset_2D(ReciprocalModifiedPowerA_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalModifiedPowerB2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Modified Power B"
        self.HTML = "y = 1.0 / a<SUP>ln(x)</SUP>"
        self.coefficientDesignatorTuple = ("a")

        self.function_cpp_code = 'temp = 1.0 / pow(coeff[0], indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_LogX(NameOrValueFlag=1), []])
    
    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / pow(a, log(x_in));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / Math.pow(a, Math.log(x_in));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / Math.Pow(a, Math.Log(x_in));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / math.pow(a, math.log(x_in))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / a ^ log(x_in)\n"
        return s


class ReciprocalModifiedPowerB_WO_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_LinearDecay_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalModifiedPowerB_ExponentialDecay_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalModifiedPowerB_LinearGrowth_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalModifiedPowerB_ExponentialGrowth_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalModifiedPowerB_LinearDecayAndOffset_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_ExponentialDecayAndOffset_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_LinearGrowthAndOffset_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_ExponentialGrowthAndOffset_2D(ReciprocalModifiedPowerB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalModifiedPowerB_Transform2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Modified Power B Transform"
        self.HTML = "y = 1.0 / a<SUP>ln(bx + c)</SUP>"
        self.coefficientDesignatorTuple = ("a", 'b', 'c')

        self.function_cpp_code = 'temp = 1.0 / pow(coeff[0], coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
    
    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / pow(a, log(b * x_in + c));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / Math.pow(a, Math.log(b * x_in + c));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / Math.Pow(a, Math.Log(b * x_in + c));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / math.pow(a, math.log(b * x_in + c))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / a ^ log(b * x_in + c)\n"
        return s


class ReciprocalModifiedPowerB_Transform_WO_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_LinearDecay_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_ExponentialDecay_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_LinearGrowth_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_ExponentialGrowth_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_LinearDecayAndOffset_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_ExponentialDecayAndOffset_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_LinearGrowthAndOffset_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerB_Transform_ExponentialGrowthAndOffset_2D(ReciprocalModifiedPowerB_Transform2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class ReciprocalModifiedPowerC2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Reciprocal Modified Power C"
        self.HTML = "y = 1.0 / ((a + x)<SUP>b</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b")

        self.function_cpp_code = 'temp = 1.0 / (pow(coeff[0] + indep_data[(int)cachedWeaveOffsets[0] + i], coeff[1]));// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
    
    def SpecificCodeCPP(self):
        s = "\ttemp = 1.0 / (pow(a + x_in, b));\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = 1.0 / (Math.pow(a + x_in, b));\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = 1.0 / (Math.Pow(a + x_in, b));\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = 1.0 / (math.pow(a + x_in, b))\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = 1.0 / ((a + x_in) ^ b)\n"
        return s


class ReciprocalModifiedPowerC_WO_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class ReciprocalModifiedPowerC_LinearDecay_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class ReciprocalModifiedPowerC_ExponentialDecay_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class ReciprocalModifiedPowerC_LinearGrowth_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class ReciprocalModifiedPowerC_ExponentialGrowth_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class ReciprocalModifiedPowerC_LinearDecayAndOffset_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerC_ExponentialDecayAndOffset_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerC_LinearGrowthAndOffset_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class ReciprocalModifiedPowerC_ExponentialGrowthAndOffset_2D(ReciprocalModifiedPowerC2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()
