#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id: YieldDensity.py 42 2008-12-13 15:48:47Z zunzun.com $
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import pythonequations, pythonequations.EquationBaseClasses
import numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them

class RationalA2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Rational A"
        self.HTML = "y = (a / (1.0 + bx)) + (c / (1.0 + dx))"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d")

        self.function_cpp_code = 'temp = coeff[0] / (1.0 + coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i]) + coeff[2] / (1.0 + coeff[3] * indep_data[(int)cachedWeaveOffsets[0] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp = a / (1.0 + b*x_in) + c / (1.0 + d*x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a / (1.0 + b*x_in) + c / (1.0 + d*x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a / (1.0 + b*x_in) + c / (1.0 + d*x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a / (1.0 + b*x_in) + c / (1.0 + d*x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a / (1.0 + b*x_in) + c / (1.0 + d*x_in)\n"
        return s


class RationalA_WO_2D(RationalA2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class RationalA_LinearDecay_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class RationalA_ExponentialDecay_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class RationalA_LinearGrowth_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class RationalA_ExponentialGrowth_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class RationalA_LinearDecayAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class RationalA_ExponentialDecayAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class RationalA_LinearGrowthAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class RationalA_ExponentialGrowthAndOffset_2D(RationalA2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()


class RationalB2D(pythonequations.EquationBaseClasses.Equation2D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation2D.__init__(self)
        self.name = "Rational B"
        self.HTML = "y = (a / (1.0 + bx + cx<sup>2</sup>)) + (d / (1.0 + ex + fx<sup>2</sup>))"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f")

        self.function_cpp_code = 'temp = coeff[0] / (1.0 + coeff[1] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[1] + i]) + coeff[3] / (1.0 + coeff[4] * indep_data[(int)cachedWeaveOffsets[0] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[1] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp = a / (1.0 + b*x_in + c*c_in*x_in) + d / (1.0 + e*x_in + f*x_in*x_in);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp = a / (1.0 + b*x_in + c*c_in*x_in) + d / (1.0 + e*x_in + f*x_in*x_in);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp = a / (1.0 + b*x_in + c*c_in*x_in) + d / (1.0 + e*x_in + f*x_in*x_in);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp = a / (1.0 + b*x_in + c*c_in*x_in) + d / (1.0 + e*x_in + f*x_in*x_in)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = a / (1.0 + b*x_in + c*c_in*x_in) + d / (1.0 + e*x_in + f*x_in*x_in)\n"
        return s


class RationalB_WO_2D(RationalB2D):
    def __init__(self): self.InitCodeForOffsetFormsOfEquations()
class RationalB_LinearDecay_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayFormsOfEquations()
class RationalB_ExponentialDecay_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayFormsOfEquations()
class RationalB_LinearGrowth_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthFormsOfEquations()
class RationalB_ExponentialGrowth_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthFormsOfEquations()
class RationalB_LinearDecayAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearDecayAndOffsetFormsOfEquations()
class RationalB_ExponentialDecayAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialDecayAndOffsetFormsOfEquations()
class RationalB_LinearGrowthAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_LinearGrowthAndOffsetFormsOfEquations()
class RationalB_ExponentialGrowthAndOffset_2D(RationalB2D):
    def __init__(self): self.InitCodeFor_X_ExponentialGrowthAndOffsetFormsOfEquations()

