#    pythonequations is a collection of equations expressed as Python classes
#    Copyright (C) 2008 James R. Phillips
#    2548 Vera Cruz Drive
#    Birmingham, AL 35235 USA
#    email: zunzun@zunzun.com
#
#    Version info: $Id$
#
#    License: BSD-style (see LICENSE.txt in main source directory)

import pythonequations, pythonequations.EquationBaseClasses
import numpy
numpy.seterr(all = 'raise') # numpy raises warnings, convert to exceptions to trap them

class Linear3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Linear"
        self.HTML = "z = a + bx + cy"
        self.coefficientDesignatorTuple = ("a", "b", "c")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        return s


class ReciprocalLinear3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Reciprocal Linear"
        self.HTML = "z = 1.0 / (a + bx + cy)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = 1.0 / temp\n"
        return s


class InverseLinear3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Inverse Linear"
        self.HTML = "z = xy / (a + bx + cy)"
        self.coefficientDesignatorTuple = ("a", "b", "c")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[3] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s


class SimplifiedQuadratic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Simplified Quadratic"
        self.HTML = "z = a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        return s


class ReciprocalSimplifiedQuadratic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Reciprocal Simplified Quadratic"
        self.HTML = "z = 1.0 / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s


class InverseSimplifiedQuadratic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Inverse Simplified Quadratic"
        self.HTML = "z = xy / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[5] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s


class FullQuadratic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Full Quadratic"
        self.HTML = "z = a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fxy"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * x_in * y_in\n"
        return s


class ReciprocalFullQuadratic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Reciprocal Full Quadratic"
        self.HTML = "z = 1.0 / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fxy)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * x_in * y_in\n"
        s += "\ttemp = 1.0 / temp\n"
        return s



class InverseFullQuadratic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Inverse Full Quadratic"
        self.HTML = "z = xy / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fxy)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[5] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        s += "\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in;\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * x_in * y_in\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * x_in * y_in\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s



class SimplifiedCubic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Simplified Cubic"
        self.HTML = "z = a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f", "g")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i] + coeff[6] * indep_data[(int)cachedWeaveOffsets[6] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[3.0]), [3.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * pow(x_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * pow(y_in, 3.0);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.pow(y_in, 3.0);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.Pow(y_in, 3.0);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * math.pow(y_in, 3.0)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * (x_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[6] + " * (y_in ^ 3.0)\n"
        return s


class ReciprocalSimplifiedCubic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Reciprocal Simplified Cubic"
        self.HTML = "z = 1.0 / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f", "g")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i] + coeff[6] * indep_data[(int)cachedWeaveOffsets[6] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[3.0]), [3.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * pow(x_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * pow(y_in, 3.0);\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.pow(y_in, 3.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.Pow(y_in, 3.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * math.pow(y_in, 3.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * (x_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[6] + " * (y_in ^ 3.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s


class InverseSimplifiedCubic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Inverse Simplified Cubic"
        self.HTML = "z = xy / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f", "g")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[7] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i] + coeff[6] * indep_data[(int)cachedWeaveOffsets[6] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * pow(x_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * pow(y_in, 3.0);\n"
        s += "\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.pow(y_in, 3.0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.Pow(y_in, 3.0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * math.pow(y_in, 3.0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * (x_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[6] + " * (y_in ^ 3.0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s


class FullCubic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Full Cubic"
        self.HTML = "z = a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP> + hxy + ix<SUP>2</SUP>y + jxy<SUP>2</SUP>"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i] + coeff[6] * indep_data[(int)cachedWeaveOffsets[6] + i] + coeff[7] * indep_data[(int)cachedWeaveOffsets[7] + i] + coeff[8] * indep_data[(int)cachedWeaveOffsets[8] + i] + coeff[9] * indep_data[(int)cachedWeaveOffsets[9] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[2.0, 1.0]), [2.0, 1.0]])
        self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[1.0, 2.0]), [1.0, 2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * pow(x_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * pow(y_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[8] + " * pow(x_in, 2.0) * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * pow(y_in, 2.0);\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.pow(y_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[8] + " * Math.pow(x_in, 2.0) * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * Math.pow(y_in, 2.0);\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.Pow(y_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[8] + " * Math.Pow(x_in, 2.0) * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * Math.Pow(y_in, 2.0);\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * math.pow(y_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[8] + " * math.pow(x_in, 2.0) * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * math.pow(y_in, 2.0)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * (x_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[6] + " * (y_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[7] + " * x_in * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[8] + " * (x_in ^ 2.0) * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[9] + " * x_in * (y_in ^ 2.0)\n"
        return s


class ReciprocalFullCubic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Reciprocal Full Cubic"
        self.HTML = "z = 1.0 / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP> + hxy + ix<SUP>2</SUP>y + jxy<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")

        self.function_cpp_code = 'temp = 1.0 / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i] + coeff[6] * indep_data[(int)cachedWeaveOffsets[6] + i] + coeff[7] * indep_data[(int)cachedWeaveOffsets[7] + i] + coeff[8] * indep_data[(int)cachedWeaveOffsets[8] + i] + coeff[9] * indep_data[(int)cachedWeaveOffsets[9] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[2.0, 1.0]), [2.0, 1.0]])
        self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[1.0, 2.0]), [1.0, 2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * pow(x_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * pow(y_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[8] + " * pow(x_in, 2.0) * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * pow(y_in, 2.0);\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.pow(y_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[8] + " * Math.pow(x_in, 2.0) * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.Pow(y_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[8] + " * Math.Pow(x_in, 2.0) * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * math.pow(y_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[8] + " * math.pow(x_in, 2.0) * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * math.pow(y_in, 2.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * (x_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[6] + " * (y_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[7] + " * x_in * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[8] + " * (x_in ^ 2.0) * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[9] + " * x_in * (y_in ^ 2.0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s


class InverseFullCubic3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "Inverse Full Cubic"
        self.HTML = "z = xy / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP> + hxy + ix<SUP>2</SUP>y + jxy<SUP>2</SUP>)"
        self.coefficientDesignatorTuple = ("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")

        self.function_cpp_code = 'temp = indep_data[(int)cachedWeaveOffsets[7] + i] / (coeff[0] + coeff[1] * indep_data[(int)cachedWeaveOffsets[1] + i] + coeff[2] * indep_data[(int)cachedWeaveOffsets[2] + i] + coeff[3] * indep_data[(int)cachedWeaveOffsets[3] + i] + coeff[4] * indep_data[(int)cachedWeaveOffsets[4] + i] + coeff[5] * indep_data[(int)cachedWeaveOffsets[5] + i] + coeff[6] * indep_data[(int)cachedWeaveOffsets[6] + i] + coeff[7] * indep_data[(int)cachedWeaveOffsets[7] + i] + coeff[8] * indep_data[(int)cachedWeaveOffsets[8] + i] + coeff[9] * indep_data[(int)cachedWeaveOffsets[9] + i]);// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        self.CacheGenerationList.append([self.CG_Ones(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_X(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_Y(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[2.0]), [2.0]])
        self.CacheGenerationList.append([self.CG_PowX(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_PowY(NameOrValueFlag=1, args=[3.0]), [3.0]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])
        self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[2.0, 1.0]), [2.0, 1.0]])
        self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[1.0, 2.0]), [1.0, 2.0]])

    def SpecificCodeCPP(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * pow(x_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * pow(y_in, 2.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * pow(x_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * pow(y_in, 3.0);\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[8] + " * pow(x_in, 2.0) * y_in;\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * pow(y_in, 2.0);\n"
        s += "\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.pow(y_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[8] + " * Math.pow(x_in, 2.0) * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * Math.pow(y_in, 2.0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = "\t\ttemp += " + self.coefficientDesignatorTuple[0] + ";\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[3] + " * Math.Pow(x_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[4] + " * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[5] + " * Math.Pow(x_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[6] + " * Math.Pow(y_in, 3.0);\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[8] + " * Math.Pow(x_in, 2.0) * y_in;\n"
        s += "\t\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * Math.Pow(y_in, 2.0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = "\ttemp += " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[3] + " * math.pow(x_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[4] + " * math.pow(y_in, 2.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[5] + " * math.pow(x_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[6] + " * math.pow(y_in, 3.0)\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[7] + " * x_in * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[8] + " * math.pow(x_in, 2.0) * y_in\n"
        s += "\ttemp += " + self.coefficientDesignatorTuple[9] + " * x_in * math.pow(y_in, 2.0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = "\ttemp = temp + " + self.coefficientDesignatorTuple[0] + "\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[1] + " * x_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[2] + " * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[3] + " * (x_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[4] + " * (y_in ^ 2.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[5] + " * (x_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[6] + " * (y_in ^ 3.0)\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[7] + " * x_in * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[8] + " * (x_in ^ 2.0) * y_in\n"
        s += "\ttemp = temp + " + self.coefficientDesignatorTuple[9] + " * x_in * (y_in ^ 2.0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s


class Polynomial3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "User-Selectable Polynomial"
        self.HTML = "z = a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP> + ..."
        self.coefficientDesignatorTuple = ()
        self.polynomialFlag = 1
        self.LinearSSQSolverFlag = 1

        self.function_cpp_code = 'temp = 0.0;\nfor (int k=0; k<numberOfCoefficients; k++)\n\ttemp += coeff[k] * indep_data[(int)cachedWeaveOffsets[k] + i];// ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[float(i), float(j)]), [float(i), float(j)]])

    def SpecificCodeCPP(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0) * pow(y_in, " + `j` + ".0);\n"
        return s

    def SpecificCodeJAVA(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * Math.pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0) * Math.pow(y_in, " + `j` + ".0);\n"
        return s

    def SpecificCodeCS(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * Math.Pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0) * Math.Pow(y_in, " + `j` + ".0);\n"
        return s

    def SpecificCodePYTHON(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + "\n"
                elif i == 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in\n"
                elif i > 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0)\n"
                elif i > 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0) * y_in\n"
                elif i == 0 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in\n"
                elif i == 0 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(y_in, " + `j` + ".0)\n"
                elif i == 1 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * math.pow(y_in, " + `j` + ".0)\n"
                elif i == 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in\n"
                else:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0) * math.pow(y_in, " + `j` + ".0)\n"
        return s

    def SpecificCodeSCILAB(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + "\n"
                elif i == 1 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in\n"
                elif i > 1 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0)\n"
                elif i > 1 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0) * y_in\n"
                elif i == 0 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * y_in\n"
                elif i == 0 and j > 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (y_in ^ " + `j` + ".0)\n"
                elif i == 1 and j > 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in * (y_in ^ " + `j` + ".0)\n"
                elif i == 1 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in * y_in\n"
                else:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0) * (y_in ^ " + `j` + ".0)\n"
        return s

    def RegenerateForm(self):
        self.coefficientDesignatorTuple = ()
        self.HTML = "z = "
        indexmax = (self.xPolynomialOrder+1) * (self.yPolynomialOrder+1)
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[index]),
                if index == 0:
                    self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[index]
                else:
                    self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[index] + 'x<SUP>' + `i` + '</SUP>y<SUP>' + `j` + '</SUP>'
                if (i+1)*(j+1) != indexmax:
                    self.HTML += ' + '

    def Initialize(self):
        self.coefficientDesignatorTuple = ()
        indexmax = (self.xPolynomialOrder+1) * (self.yPolynomialOrder+1)
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[index]),
        pythonequations.EquationBaseClasses.Equation3D.Initialize(self)


class ReciprocalPolynomial3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "User-Selectable Reciprocal Polynomial"
        self.HTML = "z = 1.0 / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP> + ...)"
        self.coefficientDesignatorTuple = ()
        self.polynomialFlag = 1

        self.function_cpp_code = 'temp = 0.0;\nfor (int k=0; k<numberOfCoefficients; k++)\n'
        self.function_cpp_code += '\ttemp += coeff[k] * indep_data[(int)cachedWeaveOffsets[k] + i];\n'
        self.function_cpp_code += 'temp = 1.0 / temp; // ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[float(i), float(j)]), [float(i), float(j)]])

    def SpecificCodeCPP(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0) * pow(y_in, " + `j` + ".0);\n"
        s += "\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * Math.pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0) * Math.pow(y_in, " + `j` + ".0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * Math.Pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0) * Math.Pow(y_in, " + `j` + ".0);\n"
        s += "\t\ttemp = 1.0 / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + "\n"
                elif i == 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in\n"
                elif i > 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0)\n"
                elif i > 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0) * y_in\n"
                elif i == 0 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in\n"
                elif i == 0 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(y_in, " + `j` + ".0)\n"
                elif i == 1 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * math.pow(y_in, " + `j` + ".0)\n"
                elif i == 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in\n"
                else:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0) * math.pow(y_in, " + `j` + ".0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + "\n"
                elif i == 1 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in\n"
                elif i > 1 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0)\n"
                elif i > 1 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0) * y_in\n"
                elif i == 0 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * y_in\n"
                elif i == 0 and j > 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (y_in ^ " + `j` + ".0)\n"
                elif i == 1 and j > 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in * (y_in ^ " + `j` + ".0)\n"
                elif i == 1 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in * y_in\n"
                else:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0) * (y_in ^ " + `j` + ".0)\n"
        s += "\ttemp = 1.0 / temp\n"
        return s

    def RegenerateForm(self):
        self.coefficientDesignatorTuple = ()
        indexmax = (self.xPolynomialOrder+1) * (self.yPolynomialOrder+1)
        self.HTML = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[index]),
                if index == 0:
                    self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[index]
                else:
                    self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[index] + 'x<SUP>' + `i` + '</SUP>y<SUP>' + `j` + '</SUP>'
                if (i+1)*(j+1) != indexmax:
                    self.HTML += ' + '
        self.HTML = "z = 1.0 / (" + self.HTML + ")"

    def Initialize(self):
        self.coefficientDesignatorTuple = ()
        indexmax = (self.xPolynomialOrder+1) * (self.yPolynomialOrder+1)
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[index]),
        pythonequations.EquationBaseClasses.Equation3D.Initialize(self)


class InversePolynomial3D(pythonequations.EquationBaseClasses.Equation3D):

    def __init__(self):
        pythonequations.EquationBaseClasses.Equation3D.__init__(self)
        self.name = "User-Selectable Inverse Polynomial"
        self.HTML = "z = xy / (a + bx + cy + dx<SUP>2</SUP> + ey<SUP>2</SUP> + fx<SUP>3</SUP> + gy<SUP>3</SUP> + ...)"
        self.coefficientDesignatorTuple = ()
        self.polynomialFlag = 1

        self.function_cpp_code = 'temp = 0.0;\nfor (int k=0; k<numberOfCoefficients; k++)\n'
        self.function_cpp_code += '\ttemp += coeff[k] * indep_data[(int)cachedWeaveOffsets[k] + i];\n'
        self.function_cpp_code += 'temp = indep_data[(int)cachedWeaveOffsets[numberOfCoefficients] + i] / temp; // ' + str(self.__class__).split(".")[len(str(self.__class__).split("."))-1]


    def CreateCacheGenerationList(self):
        self.CacheGenerationList = []
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                self.CacheGenerationList.append([self.CG_PowXY(NameOrValueFlag=1, args=[float(i), float(j)]), [float(i), float(j)]])
        self.CacheGenerationList.append([self.CG_XY(NameOrValueFlag=1), []])

    def SpecificCodeCPP(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * pow(x_in, " + `i` + ".0) * pow(y_in, " + `j` + ".0);\n"
        s += "\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeJAVA(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * Math.pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.pow(x_in, " + `i` + ".0) * Math.pow(y_in, " + `j` + ".0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodeCS(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + ";\n"
                elif i == 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in;\n"
                elif i > 1 and j == 0:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0);\n"
                elif i > 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0) * y_in;\n"
                elif i == 0 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in;\n"
                elif i == 0 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j > 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * Math.Pow(y_in, " + `j` + ".0);\n"
                elif i == 1 and j == 1:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in;\n"
                else:
                    s += "\t\ttemp += " + self.coefficientDesignatorTuple[index] + " * Math.Pow(x_in, " + `i` + ".0) * Math.Pow(y_in, " + `j` + ".0);\n"
        s += "\t\ttemp = x_in * y_in / temp;\n"
        return s

    def SpecificCodePYTHON(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + "\n"
                elif i == 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in\n"
                elif i > 1 and j == 0:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0)\n"
                elif i > 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0) * y_in\n"
                elif i == 0 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * y_in\n"
                elif i == 0 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(y_in, " + `j` + ".0)\n"
                elif i == 1 and j > 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * math.pow(y_in, " + `j` + ".0)\n"
                elif i == 1 and j == 1:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * x_in * y_in\n"
                else:
                    s += "\ttemp += " + self.coefficientDesignatorTuple[index] + " * math.pow(x_in, " + `i` + ".0) * math.pow(y_in, " + `j` + ".0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def SpecificCodeSCILAB(self):
        s = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                if i == 0 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + "\n"
                elif i == 1 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in\n"
                elif i > 1 and j == 0:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0)\n"
                elif i > 1 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0) * y_in\n"
                elif i == 0 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * y_in\n"
                elif i == 0 and j > 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (y_in ^ " + `j` + ".0)\n"
                elif i == 1 and j > 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in * (y_in ^ " + `j` + ".0)\n"
                elif i == 1 and j == 1:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * x_in * y_in\n"
                else:
                    s += "\ttemp = temp + " + self.coefficientDesignatorTuple[index] + " * (x_in ^ " + `i` + ".0) * (y_in ^ " + `j` + ".0)\n"
        s += "\ttemp = x_in * y_in / temp\n"
        return s

    def RegenerateForm(self):
        self.coefficientDesignatorTuple = ()
        indexmax = (self.xPolynomialOrder+1) * (self.yPolynomialOrder+1)
        self.HTML = ""
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[index]),
                if index == 0:
                    self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[index]
                else:
                    self.HTML += pythonequations.EquationBaseClasses.Equation.ascii[index] + 'x<SUP>' + `i` + '</SUP>y<SUP>' + `j` + '</SUP>'
                if (i+1)*(j+1) != indexmax:
                    self.HTML += ' + '
        self.HTML = "z = xy / (" + self.HTML + ")"

    def Initialize(self):
        self.coefficientDesignatorTuple = ()
        indexmax = (self.xPolynomialOrder+1) * (self.yPolynomialOrder+1)
        for i in range(0, self.xPolynomialOrder+1): # 0 - xOrder
            for j in range(0, self.yPolynomialOrder+1): # 0 - yOrder
                index = (i*(self.yPolynomialOrder+1))+j
                self.coefficientDesignatorTuple += (pythonequations.EquationBaseClasses.Equation.ascii[index]),
        pythonequations.EquationBaseClasses.Equation3D.Initialize(self)
